// cvjacke.C Version 7.41e  Copyleft (c) 杉本典夫
// Time-stamp: <2024-05-08 11:35:40 tac>
//
// 交差検証データのジャックナイフ推定値を計算する。
//
// 使用ライブラリ:libdanse.so, libfune.so
// コンパイル法  :g++ -O -s -W cvjacke cvjacke.C
//               -I/usr/local/include -L/usr/local/lib -ldanse -lfune

#include <danse.H>              // dans用ヘッダファイル(fune.H,mate.H含む)

using namespace dans;           // dans名前空間使用(名前空間matvec含む)

const vecstr USAGE =            // プログラム使用法表示用
{
  "cvjacke  Version 7.41e       Copyleft(Vo\\oV) 2024 Sugimoto Norio",
  "USAGE:cvjacke [<OPTION> ...] [<DATA FILE> ...]",
  "  Compute jackknife estimates for cross-validation data.",
  "<OPTION>",
  "  -cc[=<C-COEFFICIENT>]:confidence interval(0=<C-COEFFICIENT<1)",
  "  -f=<FILE>:reads options from the specified file",
  "  -h[=<MAN COMMAND>]:show online manual(-h:-h=man)",
  "  -r=<NUMBER>:specify the number of data in validation group"
};

const string Title = "=== Cross-Validation ==="; // 表題

class option                    // オプションクラス
{
private:
  string vpath;                 // プログラムのパス
  string vpname;                // プログラム名
  double vcc;                   // 信頼係数
  int vr;                       // 検証群のデータ数
public:
  option(const char* c_pname = nullptr); // コンストラクタ
  ~option()                              // デストラクタ
  {}
  const string& path() const    // プログラムのパスを返す
  { return vpath; }
  const string& pname() const   // プログラム名を返す
  { return vpname; }
  double cc() const             // 信頼係数を返す
  { return vcc; }
  int r() const                 // 検証群のデータ数を返す
  { return vr; }
  void setdef()                 // 省略時オプション指定を設定する
  { getoptdef(*this); }
  void set(const string& optstr, char devc = OPTdev);
  // オプション指定によってオプションを設定する
  int check(size_t ni);         // オプションのチェックとセット
};

option::option(const char* c_pname) : vpath(), vpname(), vcc(CCdef), vr(1)
// コンストラクタ
{
  if (c_pname == nullptr) return; // 初期設定せず
  vpname = getname(c_pname);    // パス名なしのプログラム名取得
  vpath = c_pname;
  strsize_t idx = vpath.find(vpname);
  if (idx != string::npos) vpath.erase(idx); // プログラム名削除
}

void option::set(const string& optstr, char devc)
// オプション指定によってオプションを設定する
{
  string opt[2];                // オプションのopt[0]:項目,opt[1]:内容
  if (getopt(opt, optstr, devc) < 1) return; // オプション指定なし
  int err = 0;
  if (opt[0] == "cc")           // 信頼係数の指定
  {
    vcc = opt[1].empty() ? CCdef : strtodf(opt[1]);
    if (vcc < 0.0 || 1.0 <= vcc) err = badopt(vpname, optstr, USAGE, 0.0, 1.0);
  }
  else if (opt[0] == "f")       // オプション指定ファイル
  { err = fgetopt(*this, opt[1], true); }
  else if (opt[0] == "h")       // オンラインヘルプ表示
  { err = help(vpname, USAGE, opt[1]); }
  else if (opt[0] == "r")       // 検証群のデータ数の指定
  {
    char* ptr = nullptr;
    if (opt[1].empty()) vr = 1;
    else vr = (int)(strtol(opt[1].c_str(), &ptr, 10));
    if (vr < 1) err = badopt(vpname, optstr, USAGE, 1.0, (double)INT_MAX);
  }
  else
  { err = nonopt(vpname, optstr, USAGE); }
  if (err > 0) throw help_except("Show this help message");
  if (err < 0) throw option_error(string("Option specification error->") + optstr);
}

int option::check(size_t ni)    // オプションのチェックとセット
{
  if (ni < 2) return disperr(vpname, "At least two items are required !");
  return 0;                     // エラーなし
}

class jackest                   // ジャックナイフ推定値クラス
{
private:
  int vk;                       // サブグループ数
  double vh0;                   // 母数θ0
  double vsigh;                 // Σθi
  double vsighm;                // Σθmi
  double vsighm2;               // Σθmi2乗
public:
  explicit jackest() : vk(0), vh0(0.0), vsigh(0.0), vsighm(0.0), vsighm2(0.0)
  // デフォールトコンストラクタ
  {}
  jackest(int k0, double h00 = 0.0) : vk(k0), vh0(h00), vsigh(0.0), vsighm(0.0), vsighm2(0.0)
  // コンストラクタ
  {}
  jackest(const jackest& jackest0) : vk(jackest0.vk), vh0(jackest0.vh0), vsigh(jackest0.vsigh), vsighm(jackest0.vsighm), vsighm2(jackest0.vsighm2)
  // コピーコンストラクタ
  {}
  virtual ~jackest()            // デストラクタ
  {}
  const jackest& operator=(const jackest& jackest0); // コピー代入演算子
  int k() const                 // サブグループ数を返す
  { return vk; }
  int k(int k0)                 // サブグループ数を設定する
  { return (vk = k0); }
  double h0() const             // 母数θ0を返す
  { return vh0; }
  double h0(double h00)         // 母数θ0を設定する
  { return (vh0 = h00); }
  double sigh() const           // Σθiを返す
  { return vsigh; }
  double sighm() const          // Σθmiを返す
  { return vsighm; }
  double sighm2() const         // Σθmi2乗を返す
  { return vsighm2; }
  void sigma(double hi);        // 母数の累積集計
  double h() const              // 母数の推定値を返す
  { return vh0 * (double)vk - vsigh * (double)(vk - 1) / (double)vk; }
  double vh() const;            // 推定値の分散を返す
  int df() const                // 自由度を返す
  { return (vk < 1) ? 0 : vk - 1; }
};

const jackest& jackest::operator=(const jackest& jackest0)
// コピー代入演算子
{
  if (&jackest0 == this) return *this; // 自分自身への代入を防ぐ
  vk = jackest0.vk;  vh0 = jackest0.vh0;
  vsigh = jackest0.vsigh;  vsighm = jackest0.vsighm;
  vsighm2 = jackest0.vsighm2;
  return *this;
}

void jackest::sigma(double hi)  // 母数の累積集計
{
  vsigh += hi;
  double hmi = (double)vk * vh0 - (double)(vk - 1) * hi;
  vsighm += hmi;  vsighm2 += hmi * hmi;
}

double jackest::vh() const      // 推定値の分散を返す
{  
  if (vk < 2) return 0.0;
  double v = (vsighm2 - vsighm * vsighm / (double)vk) / (double)vk / (double)(vk - 1);
  const double Veps = DBL_EPSILON * fmax(5.0, vsighm2);
  return (v < Veps) ? 0.0 : v;
}

class varclass                  // 変数クラス
{
private:
  size_t vnp;                   // パラメータ数
  size_t vnd;                   // 予測数
  vector<jackest> vhjkvec;      // ジャックナイフ推定値ベクトル
  vecint vn0vec;                // 全体の合計例数ベクトル
  vecint vnt0vec;               // 全体の正判別例数ベクトル
  vecint vnvec;                 // 検証結果の合計例数ベクトル
  vecint vntvec;                // 検証結果の正判別例数ベクトル
public:
  varclass(int np, int nd) : vnp(np), vnd(nd), vhjkvec(np), vn0vec(nd, 0), vnt0vec(nd, 0), vnvec(nd, 0), vntvec(nd, 0)
  // コンストラクタ
  {}
  ~varclass()                   // デストラクタ
  {}
  size_t np() const             // パラメータを返す
  { return vnp; }
  size_t nd() const             // 予測数を返す
  { return vnd; }
  void hjksigma(size_t idx, double hi) // 母数を累積する
  { if (idx < vnp) vhjkvec[idx].sigma(hi); }
  int hjkk(size_t idx) const    // サブグループ数を返す
  { return (idx < vnp) ? vhjkvec[idx].k() : 0; }
  int hjkk(size_t idx, int k0)  // サブグループ数をセットする
  { return (idx < vnp) ? vhjkvec[idx].k(k0) : 0.0; }
  double hjkh0(size_t idx) const // 母数θ0を返す
  { return (idx < vnp) ? vhjkvec[idx].h0() : 0.0; }
  double hjkh0(size_t idx, double h00) // 母数θ0をセットする
  { return (idx < vnp) ? vhjkvec[idx].h0(h00) : 0.0; }
  double hjkh(size_t idx) const // 母数推定値を返す
  { return (idx < vnp) ? vhjkvec[idx].h() : 0.0; }
  double hjkvh(size_t idx) const // 推定値の分散を返す
  { return (idx < vnp) ? vhjkvec[idx].vh() : 0.0; }
  int hjkdf(size_t idx) const   // 自由度を返す
  { return (idx < vnp) ? vhjkvec[idx].df() : 0; }
  int n0(size_t idx) const      // 全体の合計例数を返す
  { return (idx < vnd) ? vn0vec[idx] : 0; }
  int n0(size_t idx, int n00)   // 全体の合計例数をセットする
  { return (idx < vnd) ? (vn0vec[idx] = n00) : 0; }
  int nt0(size_t idx) const     // 全体の正判別例数を返す
  { return (idx < vnd) ? vnt0vec[idx] : 0; }
  int nt0(size_t idx, int nt00) // 全体の正判別例数をセットする
  { return (idx < vnd) ? (vnt0vec[idx] = nt00) : 0; }
  int n(size_t idx) const       // 検証結果の合計例数を返す
  { return (idx < vnd) ? vnvec[idx] : 0; }
  int n(size_t idx, int n0)     // 検証結果の合計例数をセットする
  { return (idx < vnd) ? (vnvec[idx] = n0) : 0; }
  int nt(size_t idx) const      // 検証結果の正判別例数を返す
  { return (idx < vnd) ? vntvec[idx] : 0; }
  int nt(size_t idx, int nt0)   // 検証結果の正判別例数をセットする
  { return (idx < vnd) ? (vntvec[idx] = nt0) : 0; }
};

class cvjackclass               // cvjackメインクラス
{
private:
  string fname;                 // ファイル名
  idd id;                       // データ識別情報
  ldata datalst;                // データリスト
  void cal_jack(varclass& var);
  // ジャックナイフ推定値を計算する
  void put_jack(ostream& fout, const option& opt, varclass& var);
  // ジャックナイフ推定値を出力する
public:
  cvjackclass() : fname(), id(), datalst() // コンストラクタ
  {}
  ~cvjackclass()                // デストラクタ
  {}
  int get(const char* c_fname, option& opt); // データを読み込み累積する
  void put(option& opt);                     // 集計結果を出力する
};

void cvjackclass::cal_jack(varclass& var)
// ジャックナイフ推定値を計算する
{
  ldata_itr dpos0 = datalst.begin(); // 最初のデータは全体の検証結果
  int k = id.nc() - 1;             // サブグループ数
  for (size_t i = 0; i < var.np(); ++i)
  {
    var.hjkk(i, k);
    var.hjkh0(i, dpos0->fdata(i + 1));
  }
  for (size_t i = 0; i < var.nd(); ++i)
  {
    size_t j = var.np() + i * 2;
    var.n0(i, (int)(dpos0->fdata(j + 1)));
    var.nt0(i, (int)(dpos0->fdata(j + 2)));
  }
  ++dpos0;
  for (ldata_itr dpos = dpos0; dpos != datalst.end(); ++dpos)
  {
    for (size_t i = 0; i < var.np(); ++i) var.hjksigma(i, dpos->fdata(i + 1));
    for (size_t i = 0; i < var.nd(); ++i)
    {
      size_t j = var.np() + i * 2;
      var.n(i, var.n(i) + (int)(dpos->fdata(j + 1)));
      var.nt(i, var.nt(i) + (int)(dpos->fdata(j + 2)));
    }
  }
}

void cvjackclass::put_jack(ostream& fout, const option& opt, varclass& var)
// ジャックナイフ推定値を出力する
{
  puttitle(fout, Title);
  fout << "\n";
  id.put_title(fout);
  id.put_stra(fout);
  fout << "<OVERALL VERIFICATION RESULTS>\n";
  for (size_t i = 0; i < var.nd(); ++i)
  {
    size_t j = var.np() + i * 2;
    double pro = (var.n0(i) > 0) ? (double)(var.nt0(i)) / (double)(var.n0(i)) * 100.0 : 0.0;
    fout << id.item(j + 1) << "=" << var.n0(i) << "\n";
    fout << id.item(j + 2) << "=" << var.nt0(i);
    fout << "(True Rate=" << pro << "%)\n";
  }
  if (opt.r() > 1)
  {
    fout << "\n<K-FOLD CROSS-VALIDATION>\n";
    fout << "  Number of Data in Validation Group=" << opt.r();
  }
  else
  { fout << "\n<LEAV-ONE-OUT CROSS-VALIDATION(LOOCV)>\n"; }
  fout << "  Number of Validations=" << var.hjkk(0) << "\n";
  for (size_t i = 0; i < var.nd(); ++i)
  {
    size_t j = var.np() + i * 2;
    double pro = (var.n(i) > 0) ? (double)(var.nt(i)) / (double)(var.n(i)) * 100.0 : 0.0;
    fout << id.item(j + 1) << "=" << var.n(i) << "\n";
    fout << id.item(j + 2) << "=" << var.nt(i);
    fout << "(True Rate=" << pro << "%)\n";
  }
  fout << "\n<JACKKNIFE ESTIMATES>\n";
  double t = ptot(1.0 - opt.cc(), var.hjkdf(0));
  for (size_t i = 0; i < var.np(); ++i)
  {
    double me = var.hjkh(i);
    double sei = (sgn(var.hjkvh(i)) > 0) ? sqrt(var.hjkvh(i)) : 0.0;
    double ci = t * sei;
    fout << id.item(i + 1) << "=" << me;
    double ml = me - ci, mu = me + ci;
    fout << " (" << opt.cc() * 100.0 << "%CI:" << ml << " - " << mu << ")" << "\n";
  }
}

int cvjackclass::get(const char* c_fname, option& opt) // データを読み込む
{
  if (c_fname == nullptr) fname.erase();
  else fname = c_fname;
  int nc = fgetdata(fname, id, datalst);
  if (nc < 0) throw runtime_error("Run-time error !");
  if (opt.check(id.ni()) != 0) throw option_error("Run-time option error !");
  nc = id.nc();                 // ケース数変更用
  for (ldata_itr dpos = datalst.begin(); dpos != datalst.end(); ++dpos)
  {
    bool blank = false;
    for (size_t j = 0; blank == false && j < id.ni(); ++j)
    {
      if (dpos->fdata(j) == BLANKf) blank = true;
    }
    if (blank)                  // ケースを削除する
    {
      dpos = datalst.erase(dpos);
      --dpos;  --nc; // 削除した次の要素の位置を返すので1つ戻す必要がある
    }
  }
  if (nc < (int)(id.nc())) id.nc(nc); // ケース数を減らす
  return nc;
}

void cvjackclass::put(option& opt) // 集計結果を出力する
{
  ldata_itr dpos0 = datalst.begin();
  int np = (int)(dpos0->fdata(0)); // パラメータ数
  int nd = (id.ni() - np) / 2; // 予測数
  varclass var(np, nd);        // 変数オブジェクト
  cal_jack(var);
  put_jack(cout, opt, var);
  cout << "\n" << endl;
}

int main(int argc, char* argv[]) // メインルーチン
{
  cvjackclass cvjack;           // メインクラス変数
  option opt(argv[0]);          // オプション
  try
  {
    opt.setdef();               // オプションの初期設定
    bool fok = true;            // ファイル処理フラッグ
    for (int i = 1; i <= argc; ++i)
    {
      if (i >= argc)
      {
        if (fok) cvjack.get(nullptr, opt); // 標準入力から読み込む
      }
      else if (*(argv[i]) == CHRopt)
      { opt.set(argv[i] + 1);  fok = true; }
      else
      { cvjack.get(argv[i], opt);  fok = false; }
    }
    cvjack.put(opt);
  }
  catch (help_except& helpex)   // ヘルプファイル表示
  { return 0; }
  catch (option_error& opterr)  // オプション指定エラー
  { return 1; }
  catch (runtime_error& rterr)  // 実行時エラー
  {
    disperr(opt.pname(), rterr.what());
    return 2;
  }
  catch (...)                   // 予期せぬ例外
  {
    disperr(opt.pname(), "unknown error occured !");
    return 255;
  }
  return 0;
}
