// designe.C  Version 7.41e  Copyleft (c) 杉本典夫
// Time-stamp: <2024-05-08 13:57:00 tac>
//
// 色々な条件から実験のパラメーターを求める。
//
// 使用ライブラリ:libdanse.so, libfune.so
// コンパイル法  :g++ -O -s -W designe designe.C
//               -I/usr/local/include -L/sur/local/lib -ldanse -lfune

#include <danse.H>              // dans用ヘッダファイル(fune.H含む)

using namespace dans;           // dans名前空間使用(名前空間matvec含む)

const vecstr USAGE =            // プログラム使用法表示用
{
  "designe  Version 7.41e       Copyleft(Vo\\oV) 2024 Sugimoto Norio",
  "USAGE:design",
  "  Compute the parameters of the experiment based on various conditions.",
  "OPTION:non"
};

const string Title = "=== Parameteres of the Experiment ==="; // 表題
constexpr double Pmin = (double)FLT_MIN;               // 最小確率値
constexpr double Pmax = 1.0 - Pmin;                    // 最大確率値

class option                    // オプションクラス
{
private:
  string vpath;                 // プログラムのパス
  string vpname;                // プログラム名
public:
  option(const char* c_pname = nullptr); // コンストラクタ
  ~option()                              // デストラクタ
  {}
  const string& path() const    // プログラムのパスを返す
  { return vpath; }
  const string& pname() const   // プログラム名を返す
  { return vpname; }
  void setdef()                 // 省略時オプション指定を設定する
  { getoptdef(*this); }
  void set(const string& optstr, char devc = OPTdev);
  // オプション指定によってオプションを設定する
};

option::option(const char* c_pname) : vpath(), vpname()
// コンストラクタ
{
  if (c_pname == nullptr) return; // 初期設定せず
  vpname = getname(c_pname);    // パス名なしのプログラム名取得
  vpath = c_pname;
  strsize_t idx = vpath.find(vpname);
  if (idx != string::npos) vpath.erase(idx); // プログラム名削除
}

void option::set(const string& optstr, char devc)
// オプション指定によってオプションを設定する
{
  string opt[2];                // オプションのopt[0]:項目,opt[1]:内容
  if (getopt(opt, optstr, devc) < 1) return; // オプション指定なし
  int err = 0;
  if (opt[0] == "f")            // オプション指定ファイル
  { err = fgetopt(*this, opt[1], true); }
  else if (opt[0] == "h")       // オンラインヘルプ表示
  { err = help(vpname, USAGE, opt[1]); }
  else                          // 間違ったオプション指定
  { err = nonopt(vpname, optstr, USAGE); }
  if (err > 0) throw help_except("Show this message");
  if (err < 0) throw option_error(string("Option specification error->") + optstr);
}

class designclass               // designメインクラス
{
private:
  double get_fdata(const string& message, double low, double up, double def) const;
  // 実数を入力する
  double sqrtn(double x, int n) const; // n次根を返す
  int sel_method(const option& opt, const string& title, int kind, int method) const;
  // 計算メニュー表示と選択
  void n_ttest() const;         // 計量値(t検定)の必要例数計算
  void n_tmcomp() const;        // 計量値(多重比較)の必要例数計算
  void n_ctest(const option& opt) const; // 計数値(χ2乗検定)の必要例数計算
  void n_cmcomp(const option& opt) const; // 計数値(多重比較)の必要例数計算
  void put_tn(int g, double za, double z2b, double sd, double d) const;
  // 計量値(多重比較)の例数出力
  void put_cn(int g, double pi1, double pi2, double za, double z2b) const;
  // 計数値(多重比較)の例数出力
  void n_mcnemar(const option& opt) const; // McNemarの検定の必要例数計算
  void n_cohort() const;
  // 症例対照研究→コホート研究(χ2乗検定)の必要例数計算
  void n_genom(const option& opt) const; // ゲノム疫学(χ2乗検定)の必要例数計算
  void n_w1test() const;   // Wilcoxonの符号付き順位検定の必要例数計算
  void n_utest() const;    // Mann-WhitneyのU検定の必要例数計算
  void n_catest() const;   // Cochran-Armitageの傾向検定の必要例数計算
  void n_cor(const option& opt) const;
  // 相関係数(Pearson、Spearman、四分点相関係数)の必要例数計算
  void n_life(const option& opt) const;
  // 生存時間の比較(単変量生存時間解析)の必要例数計算
  void n_mci() const;  // 計量値の信頼区間の必要例数計算
  void n_rci() const;  // 計数値の信頼区間の必要例数計算
  void n_icc() const;  // 級内相関係数の信頼区間の必要例数計算
  void n_kappa() const;         // 一致係数κの信頼区間の必要例数計算
  void n_hazard() const;        // ハザードλの信頼区間の必要例数計算
  void n_lracl() const;         // 回帰直線の信頼限界の必要例数計算
  void po_ttest() const;        // 計量値(t検定)の検出力計算
  void po_tmcomp() const;       // 計量値(多重比較)の検出力計算
  void po_ctest(const option& opt) const; // 計数値(χ2乗検定)の検出力計算
  void po_cmcomp() const;       // 計数値(多重比較)の検出力計算
  void po_life(const option& opt) const; // 生存時間解析の検出力計算
  void put_tpower(vector<statv<double>>& statvec, double sd, double ta, int df, bool bcput = true) const;
  // 計量値(多重比較)の検出力出力
  void put_cpower(vector<statv<double>>& statvec, double za, bool bcput = true) const;
  // 計数値(多重比較)の検出力出力
  void put_tpowersim(vector<statv<double>>& statvec, double a) const;
  // シミュレーションによる計量値(多重比較)の検出力出力
  void put_cpowersim(vector<statv<double>>& statvec, double a) const;
  // シミュレーションによる計数値(多重比較)の検出力出力
  void diff_ttest() const;      // 計量値(t検定)の検出差計算
  void diff_ctest() const;      // 計数値(χ2乗検定)の検出差計算
  void diff_life() const;       // 生存時間解析の検出差計算
  void absp_ttest() const;      // 計量値の信頼区間の絶対精度計算
  void absp_ctest() const;      // 計数値信頼区間の絶対精度計算
  void absp_lra() const;        // 回帰直線の信頼限界の絶対精度計算
  void aepr_binom() const;      // 有害事象が発現する確率計算
  void aepr_pici() const;       // 母出現率と絶対精度計算
public:
  designclass()                 // コンストラクタ
  {}
  ~designclass()                // デストラクタ
  {}
  int sel(int kind) const;                      // メニュー表示と選択
  void caln(const option& opt, int kind) const; // 実験必要例数の計算
  void calpower(const option& opt, int kind) const; // 検出力の計算
  void caldiff(const option& opt, int kind) const;  // 検出差の計算
  void calabsp(const option& opt, int kind) const;  // 信頼区間の絶対精度の計算
  void calaepr(const option& opt, int kind) const;
  // 有害事象が発現する確率の計算
};

double designclass::get_fdata(const string& message, double low, double up, double def) const
// 実数を入力する
{
  string sline;
  bool ok = false;
  double fdata = 0.0;
  istream* finp = nullptr;
  finp = &cin;
  do
  {
    cout << message << "(" << low << " - " << up << ",[Return]:" << def << "):";
    getsline(sline, *finp);
    strchomp(sline);
    fdata = sline.empty() ? def : strtodf(sline);
    if (fdata < low || up < fdata) beep();
    else ok = true;
  } while (!ok);
  return fdata;
}

double designclass::sqrtn(double x, int n) const // n次根を返す
{
  if (n < 0) throw runtime_error("Argument of the nth-order root function is negative !");
  if (n == 0) return 1.0;
  switch (n)
  {
  case 1: return x;
  case 2 : return sqrt(x);
  case 4 : return sqrt(sqrt(x));
  }
  return exp(log(x) / (double)n);
}

int designclass::sel_method(const option& opt, const string& title, int kind, int method) const
// 計算メニュー表示と選択
{
  double fmax = 0.0;
  cout << "--- " << title << " ---\n\n";
  switch (kind)
  {
  case 1 : fmax = 18.0;
    cout << "1:Metric Data(t-test)\n";
    cout << "2:Metric Data(Multiple Comparison)\n";
    cout << "3:Nominal Data(Chi-squared Test)\n";
    cout << "4:Nominal Data(Multiple Comparison)\n";
    cout << "5:Paired Two-sample Nominal Data(McNemar's Test)\n";
    cout << "6:Case-Control Study -> Cohort Study(Chi-squared Test)\n";
    cout << "7:Genomic Epidemiology(Chi-squared Test)\n";
    cout << "8:Wilcoxon's Signed Rank Test\n";
    cout << "9:Mann-Whitney's U-test(with winning pro=AUC of the ROC Curve)\n";
    cout << "10:Cochran-Armitage' Trend Test\n";
    cout << "11:Correlation Coefficient(Pearson,Spearman,Four-fold Point r)\n";
    cout << "12:Survival Time Analysis(PHR-test)\n";
    cout << "13:Confidence Interval of Metric Data\n";
    cout << "14:Confidence Interval of Nominal Data\n";
    cout << "15:Confidence Interval of Intraclass Correlation Coefficient\n";
    cout << "16:Confidence Interval of Matching Coefficient Kappa\n";
    cout << "17:Confidence Interval of Hazard\n";
    cout << "18:Confidence Interval of Regression Line\n";
    break;
  case 2 : fmax = 5.0;
    cout << "1:Metric Data(t-test)\n";
    cout << "2:Metric Data(Multiple Comparison)\n";
    cout << "3:Nominal Data(Chi-squared Test)\n";
    cout << "4:Nominal Data(Multiple Comparison)\n";
    cout << "5:Survival Time Analysis(PHR-test)\n";
    break;
  case 3 : fmax = 3.0;
    cout << "1:Metric Data(t-test)\n";
    cout << "2:Nominal Data(Chi-squared Test)\n";
    cout << "3:Survival Time Analysis(PHR-test)\n";
    break;
  case 4 : fmax = 3.0;
    cout << "1:Confidence Interval of Metric Data\n";
    cout << "2:Confidence Interval of Nominal Data\n";
    cout << "3:Confidence Interval of Regression Line\n";
    break;
  case 5 : fmax = 2.0;
    cout << "1:Probability of Adverse Events Occurring\n";
    cout << "2:Proportion and Absolute Precision\n";
    break;
  default : disperr(opt.pname(), "Wrong number specified !");
  }
  cout << "\n0:Return to Main Menu\n\n";
  return (int)(get_fdata("enter number", 0.0, fmax, (double)method));
}

void designclass::n_ttest() const // 計量値(t検定)の必要例数計算
{
  cout << "--- Metric Data(t-test) ---\n";
  double a = ALdef, po = POdef, d = 1.0, sd = 1.0, r = 1.0;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    d = get_fdata("significant difference(delta)", DBL_MIN, DBL_MAX, d);
    sd = get_fdata("population SD(sigma)", DBL_MIN, DBL_MAX, sd);
    r = get_fdata("ratio of n2/n1", DBL_MIN, DBL_MAX, r);
    double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
    if (po < 0.5) z2b = -ptoz(2.0 * po);
    double lamda = d / sd;
    double nsqrt = (za + z2b) / lamda;
    double n0 = (1.0 + r) / r * nsqrt * nsqrt;
    cout << "Comparison of the 2 Groups(between-group comparison)\n";
    int n1 = (int)(n0 + 0.99);
    int n2 = (int)((double)n1 * r + 0.99);
    double n = n0 + n0 * r;
    double f = (n + 1.0) / (n - 1.0);
    int nt = (int)(n * f + 0.99);
    if (nt < 1) nt = 1;
    double r2 = lamda * lamda / (lamda * lamda + (double)(nt - 2) * (double)nt / (double)n1 / (double)n2);
    cout << "  delta=" << d << "  sigma=" << sd << "  Effect Size:delta/sigma=" << lamda << "\n";
    cout << "  Correlation Ratio:r=" << sqrt(r2) << "  r-squared=" << r2 << "\n";
    cout << "  When the variance is known:n1=" << n1 << "  n2=" << n2 << "  (total n with freedom-modified=" << nt << ")\n";
    n1 = (n1 < 2) ? n1 + 1 : (int)(n0 + za * za / 4.0 / (1.0 - 1.0 / (double)n1) + 0.99);
    n2 = (int)(n1 * r + 0.99);
    cout << "  When the variance is unknown:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
    cout << "Comparison of 1 group(within-group comparison)\n";
    n0 = nsqrt * nsqrt;
    n1 = (int)(n0 + 0.99);
    cout << "  When the variance is known:n=" << n1 << "\n";
    n2 = (n1 < 2) ? n1 + 2 : (int)(n0 + za * za / 2.0 / (1.0 - 1.0 / (double)n1) + 0.99);
    cout << "  When the Variance is unknown:n=" << n2 << "\n\n";
  }
}

void designclass::n_tmcomp() const // 計量値(多重比較)の必要例数計算
{
  cout << "--- Metric Data(Multiple Comparison) ---\n";
  int g = 0, gd = 1;
  double a = ALdef, po = POdef, d = 1.0, sd = 1.0;
  while ((g = (int)(get_fdata("number of groups(0:quit)", 0.0, 100.0, (double)g))) > 1)
  {
    gd = (int)(get_fdata("number of groups with different means(0:unknown)", 0.0, (double)(g - 1), (double)gd));
    a = get_fdata("significance level(alpha)", Pmin, Pmax, a);
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    d = get_fdata("significant difference(delta)", DBL_MIN, DBL_MAX, d);
    sd = get_fdata("common SD(sigma)", DBL_MIN, DBL_MAX, sd);
    cout << "Fisher type:number of comparisons=1\n";
    cout << "  number of comparisons that become significant=1\n";
    cout << "  alpha=" << a << "  power=" << po << "\n";
    double za = ptoz(a);
    double z2b = (po > 0.5) ? ptoz(2.0 * (1.0 - po)) : -ptoz(2.0 * po);
    put_tn(g, za, z2b, sd, d);
    int nc = g * (g - 1) / 2;   // 比較回数
    int nd = (gd < 1) ? 1 : gd * (2 * g - gd - 1) / 2; // 有意になる比較回数
    double ac = a / (double)nc; // 多重性調整済αエラー
    double poc = 1.0 - sqrtn(1.0 - po, nd); // 多重性調整済検出力
    double zac = ptoz(ac);
    double z2bc = (poc > 0.5) ? ptoz(2.0 * (1.0 - poc)) : -ptoz(2.0 * poc);
    cout << "Bonferroni type:number of comparisons=" << nc << "\n";
    cout << "  number of comparisons that become significant=" << nd << "\n";
    cout << "  alpha_c=" << ac << "  power=" << poc << "\n";
    zac = ptoz(ac);
    put_tn(g, zac, z2bc, sd, d);
    cout << "Scheffe type:number of comparisons=" << nc << "\n";
    cout << "  number of comparisons that become significant=" << nd << "\n";
    cout << "  alpha=" << a << "  power=" << poc << "\n";
    zac = sqrt(ptoc(a, g - 1));
    put_tn(g, zac, z2bc, sd, d);
    ac = sqrtn(a, nd);
    poc = sqrtn(po, nc);
    cout << "Step method:number of comparisons=" << nc << "\n";
    cout << "  number of comparisons that become significant=" << nd << "\n";
    cout << "  alpha_c=" << ac << "  power=" << poc << "\n";
    zac = ptoz(ac);
    z2bc = (poc > 0.5) ? ptoz(2.0 * (1.0 - poc)) : -ptoz(2.0 * poc);
    put_tn(g, zac, z2bc, sd, d);
  }
}

void designclass::n_ctest(const option& opt) const
// 計数値(χ2乗検定)の必要例数計算
{
  cout << "--- Nominal Data(Chi-squared Test) ---\n";
  double a = ALdef, po = POdef, pi1 = 0.6, pi2 = 0.4, s = 1.0;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    pi1 = get_fdata("proportion of group1(pi1)", 0.0, 1.0, pi1);
    while ((pi2 = get_fdata("proportion of group2 or theoretical proportion(pi2)", 0.0, 1.0, pi2)) == pi1) disperr(opt.pname(), "can not compute when pi1=pi2 !");
    s = get_fdata("ratio of n2/n1", DBL_MIN, DBL_MAX, s);
    double pi = (pi1 + s * pi2) / (1.0 + s), dpi = pi1 - pi2;
    double v = pi * (1.0 - pi);
    double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
    if (po < 0.5) z2b = -ptoz(2.0 * po);
    cout << "Comparison of the 2 groups(between-group comparison)\n";
    cout << "  pi1=" << pi1 << "  pi2=" << pi2 << "  pi_d = pi1 - pi2=" << dpi << "\n";
    dpi = fabs(dpi);
    double r2 = dpi / (1.0 + s);
    r2 = s * r2 * r2 / v;
    cout << "  Coefficient of Contingency:V=" << sqrt(r2) << "  V-squared=" << r2 << "\n";
    double nsqrt = (za * sqrt((1.0 + s) * v) + z2b * sqrt(s * pi1 * (1.0 - pi1) + pi2 * (1.0 - pi2))) / dpi;
    double n0 = nsqrt * nsqrt / s;
    int n1 = (int)(n0 + 0.99);
    int n2 = (int)(n0 * s + 0.99);
    cout << "  No continuous correction:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n";
    nsqrt = 1.0 + sqrt(1.0 + (1.0 + s) / s * 2.0 / n0 / dpi);
    n1 = (int)(n0 / 4.0 * nsqrt * nsqrt + 0.99);
    n2 = (int)((double)n1 * s + 0.99);
    cout << "  With continuous correction:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
    double cs = (s * pi1 * (1.0 - pi1) + pi2 * (1.0 - pi2)) / s;
    nsqrt = (za + z2b) / dpi;
    n0 = cs * nsqrt * nsqrt;
    n1 = (int)(n0 + 0.99);
    n2 = (int)(n0 * s + 0.99);
    cout << "  No continuous correction(simple method):n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n";
    nsqrt = (za + z2b) / (dpi - sgn(dpi) * 0.5 / n0 * (1.0 + s) / s);
    n0 = cs * nsqrt * nsqrt;
    n1 = (int)(n0 + 0.99);
    n2 = (int)(n0 * s + 0.99);
    cout << "  with continuous correction(simple method):n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
    cout << "Comparison of 1 group(within-group comparison)\n";
    cout << "  pi1=" << pi1 << "  pi2=" << pi2 << "  pi_d = pi1 - pi=" << dpi << "\n";
    double v0 = pi2 * (1.0 - pi2), v1 = pi1 * (1.0 - pi1);
    nsqrt = (za * sqrt(v0) + z2b * sqrt(v1)) / dpi;
    n0 = nsqrt * nsqrt;
    cout << "  No continuous correction:n=" << (int)(n0 + 0.99) << "\n\n";
    cout << "Test of Risk Ratio(test of logarithm risk difference)\n";
    cout << "  pi1=" << pi1 << "  pi2=" << pi2 << "\n";
    double rr = (pi2 > 0.0) ? pi1 / pi2 : 1.0;
    double lrr = log(rr);
    cout << "  Risk Ratio:RR(pi1/pi2)=" << rr << "  ln(RR)=" << lrr << "\n";
    cs = (1.0 - pi1) / pi1 + (1.0 - pi2) / pi2 / s;
    nsqrt = (za + z2b) / lrr;
    n0 = cs * nsqrt * nsqrt;
    n1 = (int)(n0 + 0.99);
    n2 = (int)(n0 * s + 0.99);
    cout << "  n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
    cout << "Test of Odds Ratio(test of logarithm odds difference)\n";
    cout << "  pi1=" << pi1 << "  pi2=" << pi2 << "\n";
    double oz1 = (pi1 > 0.0) ? pi1 / (1.0 - pi1) : 1.0;
    double oz2 = (pi2 > 0.0) ? pi2 / (1.0 - pi2) : 1.0;
    double lor = log(oz1 / oz2);
    cout << "  Odds Ratio:OR(pi1/pi2)=" << oz1 / oz2 << "  ln(OR)=" << lor << "\n";
    cs = 1.0 / pi1 / (1.0 - pi1) + 1.0 / pi2 / (1.0 - pi2) / s;
    nsqrt = (za + z2b) / lor;
    n0 = cs * nsqrt * nsqrt;
    n1 = (int)(n0 + 0.99);
    n2 = (int)(n0 * s + 0.99);
    cout << "  n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
  }
}

void designclass::n_cmcomp(const option& opt) const
// 計数値(多重比較)の必要例数計算
{
  cout << "--- Nominal Data(Multiple Comparison) ---\n";
  int g = 0, gd = 1;
  double a = ALdef, po = POdef, pi1 = 0.6, pi2 = 0.5;
  while ((g = (int)(get_fdata("number of groups(0:quit)", 0.0, 100.0, (double)g))) > 1)
  {
    gd = (int)(get_fdata("number of groups with different proportions(0:unknown)", 0.0, (double)(g - 1), (double)gd));
    a = get_fdata("significance level(alpha)", Pmin, Pmax, a);
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    pi1 = get_fdata("proportion of group-i(pi_i)", 0.0, 1.0, pi1);
    while ((pi2 = get_fdata("proportion of group-j(pi_j)", 0.0, 1.0, pi2)) == pi1) disperr(opt.pname(), "can not compute when pi_i=pi_j !");
    cout << "Fisher type:number of comparisons=1\n";
    cout << "  number of comparisons that become significant=1\n";
    cout << "  alpha=" << a << "  power=" << po << "\n";
    double za = ptoz(a);
    double z2b = (po > 0.5) ? ptoz(2.0 * (1.0 - po)) : -ptoz(2.0 * po);
    put_cn(g, pi1, pi2, za, z2b);
    int nc = g * (g - 1) / 2;   // 比較回数
    int nd = (gd < 1) ? 1 : gd * (2 * g - gd - 1) / 2; // 有意になる比較回数
    double ac = a / (double)nc; // 多重性調整済αエラー
    double poc = 1.0 - sqrtn(1.0 - po, nd); // 多重性調整済検出力
     double zac = ptoz(ac);
    double z2bc = (poc > 0.5) ? ptoz(2.0 * (1.0 - poc)) : -ptoz(2.0 * poc);
    cout << "Bonferroni type:number of comparisons=" << nc << "\n";
    cout << "  number of comparisons that become significant=" << nd << "\n";
    cout << "  alpha_c=" << ac << "  power=" << poc << "\n";
    zac = ptoz(ac);
    put_cn(g, pi1, pi2, zac, z2bc);
    cout << "Scheffe type:number of comparisons=" << nc << "\n";
    cout << "  number of comparisons that become significant=" << nd << "\n";
    cout << "  alpha=" << a << "  power=" << poc << "\n";
    zac = sqrt(ptoc(a, g - 1));
    put_cn(g, pi1, pi2, zac, z2bc);
    ac = sqrtn(a, nd);
    poc = sqrtn(po, nc);
    cout << "Step method:number of comparisons=" << nc << "\n";
    cout << "  number of comparisons that become significant=" << nd << "\n";
    cout << "  alpha_c=" << ac << "  power=" << poc << "\n";
    zac = ptoz(ac);
    z2bc = (poc > 0.5) ? ptoz(2.0 * (1.0 - poc)) : -ptoz(2.0 * poc);
    put_cn(g, pi1, pi2, zac, z2bc);
  }
}

void designclass::put_tn(int g, double za, double z2b, double sd, double d) const
// 計量値(多重比較)の例数出力
{
  double nsqrt = (za + z2b) * sd / d;
  double n0 = 2.0 * nsqrt * nsqrt;
  int n = (int)(n0 + 0.99);
  cout << "  When the variance is known:n1=" << n << "  (total n=" << g * n << ")\n";
  n = (n < 2) ? n + 1 : (int)(n0 + za * za / 4.0 / (1.0 - 1.0 / (double)n) + 0.99);
  cout << "  When the variance if unknown:n1=" << n << "  (total n=" << g * n << ")\n\n";
}

void designclass::put_cn(int g, double pi1, double pi2, double za, double z2b) const
// 計数値(多重比較)の例数出力
{
  double pi = (pi1 + pi2) * 0.5, dpi = fabs(pi1 - pi2);
  double v = pi * (1.0 - pi);
  double nsqrt = (za * sqrt(2.0 * v) + z2b * sqrt(pi1 * (1.0 - pi1) + pi2 * (1.0 - pi2))) / dpi;
  double n0 = nsqrt * nsqrt;
  int n = (int)(n0 + 0.99);
  cout << "  No continuous correction:n1=" << n << "  (total n=" << g * n << ")\n";
  nsqrt = 1.0 + sqrt(1.0 + 4.0 / n0 / dpi);
  n = int(n0 / 4.0 * nsqrt * nsqrt + 0.99);
  cout << "  With continuous correction:n1=" << n << "  (total n=" << g * n << ")\n\n";
}

void designclass::n_mcnemar(const option& opt) const
// McNemarの検定の必要例数計算
{
  cout << "--- Paired Two-sample Nominal Data(McNemar's Test) ---\n";
  double a = ALdef, po = POdef, pi1 = 0.6, pi2 = 0.5;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    pi1 = get_fdata("proportion of item-1 and category-1(pi1)", 0.0, 1.0, pi1);
    while ((pi2 = get_fdata("proportion of item-2 and category-1(pi2)", 0.0, 1.0, pi2)) == pi1) disperr(opt.pname(), "can not compute when pi1=pi2 !");
    double pix = pi1 + pi2 - 2.0 * pi1 * pi2;
    double dpi = fabs(pi1 - pi2) / pix;
    double pip = 0.5 + dpi * 0.5, pim = 0.5 - dpi * 0.5;
    double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
    if (po < 0.5) z2b = -ptoz(2.0 * po);
    double n0 = (za + z2b * 2.0 * sqrt(pip * pim)) / (pip - pim);
    n0 = n0 * n0;
    double n = (int)(n0 / pix + 0.99);
    cout << "Paired Two-sample Nominal Data(McNemar's Test)\n";
    cout << "  pi1=" << pi1 << "  pi2=" << pi2 << "\n";
    cout << "  pi+=" << pip << "  pi-=" << pim << "  thoretical proportion=0.5\n";
    cout << "  McNemar's Test(no continuous correction)=" << int(n0 + 0.99) << "\n";
    cout << "  Total n=" << n << "\n\n";
  }
}

void designclass::n_cohort() const
// 症例対照研究→コホート研究(χ2乗検定)の必要例数計算
{
  cout << "--- Case-Control Study -> Cohort Study(Chi-squared Test) ---\n";
  double a = ALdef, po = POdef, pid = 0.1, sn = 0.7, sp = 0.8, s = 1.0;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    pid = get_fdata("incidence of disease(pi_d)", 0.0, 1.0, pid);
    sn = get_fdata("sensitivity of examination(sn)", 0.0, 1.0, sn);
    sp = get_fdata("specificity of examination(sp)", 0.0, 1.0, sp);
    s = get_fdata("ratio of n2/n1", DBL_MIN, DBL_MAX, s);
    double pi1 = pid * sn / (pid * sn + (1.0 - pid) * (1.0 - sp));
    double pi2 = 1.0 - (1.0 - pid) * sp / ((1.0 - pid) * sp + pid * (1.0 - sn));
    double rr = pi1 / pi2;
    double pi = (pi1 + s * pi2) / (1.0 + s), dpi = fabs(pi1 - pi2);
    double v = pi * (1.0 - pi);
    double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
    if (po < 0.5) z2b = -ptoz(2.0 * po);
    double r = sqrt(s) / (1.0 + s) * dpi / sqrt(v);
    cout << "Proporion of group1:pi1=" << pi1 << "  Proportion of group2:pi2=" << pi2 << "\n";
    cout << "  RR(pi1/pi2)=" << rr << "  Coefficient of contingency:V=" << r << "  V-squared=" << r * r << "\n";
    cout << "Comparison of the 2 groups(between-group comparison)\n";
    double nsqrt = (za * sqrt((1.0 + s) * v) + z2b * sqrt(s * pi1 * (1.0 - pi1) + pi2 * (1.0 - pi2))) / dpi;
    double n0 = nsqrt * nsqrt / s;
    int n1 = (int)(n0 + 0.99);
    int n2 = (int)((double)n1 * s + 0.99);
    cout << "  No continuous correction:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n";
    nsqrt = 1.0 + sqrt(1.0 + (1.0 + s) / s * 2.0 / n0 / dpi);
    n1 = (int)(n0 / 4.0 * nsqrt * nsqrt + 0.99);
    n2 = (int)((double)n1 * s + 0.99);
    cout << "  With continuous correction:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
    cout << "Comparison of 1 group(within-group comparison)\n";
    pi = (pi1 + pi2) / 2.0;
    nsqrt = (za * sqrt(2.0 * pi) + z2b * sqrt(2.0 * pi1 * pi2 / pi)) / dpi;
    n0 = nsqrt * nsqrt;
    cout << "  With continuous correction:n=" << (int)(n0 + 0.99) << "\n\n";
  }
}

void designclass::n_genom(const option& opt) const
// ゲノム疫学(χ2乗検定)の必要例数計算
{
  cout << "--- Genomic Epidemiology(Chi-squared Test) ---\n";
  double a = ALdef, po = POdef, pid = 0.1, pig = 0.5, rr = 2.0, s = 1.0;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    pid = get_fdata("incidence of disease(pi_d)", 0.0, 1.0, pid);
    pig = get_fdata("frequency of gene(pi_g)", 0.0, 1.0, pig);
    while ((rr = get_fdata("relative risk(RR)", DBL_MIN, DBL_MAX, rr)) == 1.0) disperr(opt.pname(), "can not compute when RR=1 !");
    s = get_fdata("ratio of n2/n1", DBL_MIN, DBL_MAX, s);
    double pi2 = pid / (pig * (rr - 1.0) + 1.0);
    double pi1 = rr * pi2;
    double odr = pi1 * (1.0 - pi2) / pi2 / (1.0 - pi1);
    double pi = (pi1 + s * pi2) / (1.0 + s), dpi = fabs(pi1 - pi2);
    double v = pi * (1.0 - pi);
    double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
    if (po < 0.5) z2b = -ptoz(2.0 * po);
    double r = sqrt(s) / (1.0 + s) * dpi / sqrt(v);
    cout << "Proportion of group1:pi1=" << pi1 << "  Proportion of group2:pi2=" << pi2 << "\n";
    cout << "  OR=" << odr << "  Coefficient of contingency:V=" << r << "  V-squared=" << r * r << "\n";
    cout << "Comparison of the 2 group(between-group comparison)\n";
    double nsqrt = (za * sqrt((1.0 + s) * v) + z2b * sqrt(s * pi1 * (1.0 - pi1) + pi2 * (1.0 - pi2))) / dpi;
    double n0 = nsqrt * nsqrt / s;
    int n1 = (int)(n0 + 0.99);
    int n2 = (int)((double)n1 * s + 0.99);
    cout << "  No continuous correction:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n";
    nsqrt = 1.0 + sqrt(1.0 + (1.0 + s) / s * 2.0 / n0 / dpi);
    n1 = (int)(n0 / 4.0 * nsqrt * nsqrt + 0.99);
    n2 = (int)((double)n1 * s + 0.99);
    cout << "  With continuous correction:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
    cout << "Comparison of 1 group(within-group comparison)\n";
    pi = (pi1 + pi2) / 2.0;
    nsqrt = (za * sqrt(2.0 * pi) + z2b * sqrt(2.0 * pi1 * pi2 / pi)) / dpi;
    n0 = nsqrt * nsqrt;
    cout << "  With continuous correction:n=" << (int)(n0 + 0.99) << "\n\n";
  }
}

void designclass::n_w1test() const // Wilcoxnの符号付き順位検定の必要例数計算
{
  cout << "--- Wilcoxon's Signed Rank Test ---\n";
  double a = ALdef, po = POdef, pi = 0.2, kr = 1.0;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    pi = get_fdata("proportion of signed rank mean", DBL_MIN, 1.0, pi);
    kr = get_fdata("correction of equivalence K", DBL_MIN, DBL_MAX, kr);
    double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
    if (po < 0.5) z2b = -ptoz(2.0 * po);
    double z = za + z2b;
    int n = (int)(kr * z * z / 3.0 / pi / pi + 0.99);
    double ts = pi * (double)n * (double)n;
    cout << "Wilcoxon's Signed Rank Test\n";
    cout << "  Proportion of signed rank mean=" << pi << "\n";
    cout << "  Signed rank mean=" << pi * (double)n << "  Signed rank sum=" << int(ts + 0.5) << "\n";
    cout << "  n(n+ + n-)=" << n << "\n\n";
  }
}

void designclass::n_utest() const // Mann-WhitneyのU検定の必要例数計算(勝率から)
{
  cout << "--- Mann-Whitney's U-test ---\n";
  double a = ALdef, po = POdef, pi1 = 0.7, s = 1.0, kr = 1.0;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    pi1 = get_fdata("winning proportion of group1(U/(2n))", 0.5 + DBL_EPSILON, 1.0, pi1);
    kr = get_fdata("correction of equivalence K", DBL_MIN, DBL_MAX, kr);
    s = get_fdata("ratio of n2/n1", DBL_MIN, DBL_MAX, s);
    double dp = pi1 - 0.5;
    double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
    if (po < 0.5) z2b = -ptoz(2.0 * po);
    double z = za + z2b;
    int n1 = (int)((s + 1.0) * kr * z * z / s / 12.0 / dp / dp + 0.99);
    int n2 = (int)((double)n1 * s + 0.99);
    double u1 = pi1 * (double)n1 * (double)n2;
    double r2 = 3.0 * dp * dp / kr;
    cout << "Mann-Whitney's U-test(test of AUC of ROC curve)\n";
    cout << "  Winning proportion of group1:U/(2n)=" << pi1 << "  U=" << (int)(u1 + 0.5) << "\n";
    cout << "  Correlation Ratio:r=" << sqrt(r2) << "  r-squared=" << r2 << "\n";
    cout << "  n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
  }
}

void designclass::n_catest() const // Cochran-Armitageの傾向検定の必要例数計算
{
  cout << "--- Cochran-Armitage' Trend Test ---\n";
  int g = 0;
  double a = ALdef, po = POdef;
  while ((g = (int)(get_fdata("number of groups(0:quit)", 0.0, 100.0, (double)g))) > 1.0)
  {
    vecdbl xvec(g, 0.0), pvec(g, 0.0);
    a = get_fdata("significance level(alpha)", Pmin, Pmax, a);
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    double p = 0.0, sx = 0.0, sx2 = 0.0;
    for (int i = 0; i < g; i++)
    {
      if (xvec[i] <= 0.0) xvec[i] = i;
      if (pvec[i] <= 0.0) pvec[i] = (double)i * 0.1;
      ostringstream ostr;
      ostr << "explanatory variable x" << i + 1 << " of group" << i + 1;
      xvec[i] = get_fdata(ostr.str(), -DBL_MAX, DBL_MAX, xvec[i]);
      sx += xvec[i];  sx2 += xvec[i] * xvec[i];
      ostr.str("");
      ostr << "responce rate-" << i + 1 << " of group" << i + 1;
      pvec[i] = get_fdata(ostr.str(), 0.0, 1.0, pvec[i]);
      p += pvec[i];
    }
    p /= (double)g;
    double u = 0.0, v = (double)g * sx2 - sx * sx;
    cout << "Cochran-Armitage's Trend Test\n";
    for (int i = 0; i < g; i++)
    {
      cout << "  x" << i + 1 << " if group" << i + 1 << "=" << xvec[i];
      cout << " p" << i + 1 << "=" << pvec[i] << "\n";
      u += xvec[i] * (pvec[i] - p);
    }
    if (p <= 0.0 || 1.0 <= p || sgn(u) == 0 || sgn(v) == 0)
    { cout << "  can not compute total number !\n\n"; }
    else
    {
      double r2 = u * u / p / (1.0 - p) / v;
      double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
      if (po < 0.5) z2b = -ptoz(2.0 * po);
      double z = za + z2b;
      int n = (int)(z * z / r2) + 1;
      int ni = (int)((double)n / (double)g + 0.99);
      cout << "  Correlation Ratio:r=" << sqrt(r2) << "  r-squared=" << r2 << "\n";
      cout << "  Total n=" << n << "  n1=" << ni << "\n\n";
    }
  }
}

void designclass::n_cor(const option& opt) const
// 相関係数(Pearson、Spearman、四分点相関係数)の必要例数計算
{
  cout << "--- Correlation Coefficient(Pearson,Spearman,Four-fold Point r) ---\n";
  double a = ALdef, po = POdef, r = 0.5, s = 1.0;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    while ((r = get_fdata("correlation coefficient(r)", 0.0, 1.0, r)) == 0.0 || r == 1.0) disperr(opt.pname(), "can not compute when ρ=0,ρ=1 !");
    s = get_fdata("ratio of n2/n1", DBL_MIN, DBL_MAX, s);
    cout << "Correlation Coefficient:r=" << r << "  r-suared=" << r * r << "\n";
    double pi2 = (2.0 - r * (1.0 + s)) / 4.0 / sqrt(s);
    double pi1 = 1.0 - pi2;
    cout << "  Proportion of group1:pi1=" << pi1 << "  Proportion of group2:pi2=" << pi2 << "\n\n";
    double z = log((1.0 + r) / (1.0 - r));
    double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
    if (po < 0.5) z2b = -ptoz(2.0 * po);
    double z0 = za + z2b;
    int n = (int)(4.0 * z0 / z * z0 / z + 0.99) + 3;
    cout << "Pearson's Correlation Coefficient:total n=" << n << "\n";
    int n1 = (int)((double)n / (1.0 + s) + 0.99);
    int n2 = (int)((double)n1 * s + 0.99);
    cout << "  Correlation Ratio:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n";
    n = (int)(z0 / r * z0 / r + 0.99) + 1;
    cout << "  Spearman's Rank Correlation Coefficient:total n=" << n << "\n";
    n1 = (int)((double)n / (1.0 + s) + 0.99);
    n2 = (int)((double)n1 * s + 0.99);
    cout << "  Rank Correlation Ratio:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n";
    cout << "  Four-fold Point r or Coefficient of Contingency:n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
  }
}

void designclass::n_life(const option& opt) const
// 生存時間の比較(単変量生存時間解析)の必要例数計算
{
  cout << "--- Survival Time Analysis ---\n";
  double a = ALdef, po = POdef, lamda1 = 0.2, lamda2 = 0.1, tmax = 0.0, s = 1.0;
  while ((a = get_fdata("siginificance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    lamda1 = get_fdata("hazard of group1(lambda1)", Pmin, 1.0, lamda1);
    while ((lamda2 = get_fdata("theoreticla hazard or hazard of group2(lambda2)", Pmin, 1.0, lamda2)) == lamda1) disperr(opt.pname(), "can not calculete when lambda1=lambda2 !");
    tmax = get_fdata("observation period(t,0:infinity)", 0.0, DBL_MAX, tmax);
    s = get_fdata("ratio of n2/n1", DBL_MIN, DBL_MAX, s);
    double hd = lamda1 - lamda2, hr = lamda1 / lamda2;
    double b1 = log(hr);
    double za = ptoz(a), z2b = ptoned(po);
    cout << "Test of Hazard Ratio(within-group comparison)\n";
    cout << "  lambda1=" << lamda1 << "  Theoretical lambda2=" << lamda2 << "\n";
    cout << "  HR(lambda1/lambda2)=" << hr << "  ln(HR)=" << b1 << "  t=";
    if (tmax > 0.0) cout << tmax << "\n";
    else cout << "infinity\n";
    double zt = (za  + z2b) / b1;
    double zt2 = zt * zt;
    double exp1 = (tmax > 0.0) ? 1.0 - exp(-lamda1 * tmax) : 1.0;
    int nd = (int)(zt2 + 0.99), n1 = (int)(zt2 / exp1 + 0.99);
    cout << "  n1=" << n1 << "  (total deaths=" << nd << ")\n\n";
    cout << "Parametric Hazard Ratio Test(between-group comparison)\n";
    cout << "  lambda1=" << lamda1 << "  lambda2=" << lamda2 << "\n";
    cout << "  HR(lambda1/lambda2)=" << hr << "  ln(HR)=" << b1 << "  t=";
    if (tmax > 0.0) cout << tmax << "\n";
    else cout << "infinity\n";
    double exp2 = (tmax > 0.0) ? 1.0 - exp(-lamda2 * tmax) : 1.0;
    double n0 = (s * exp1 + exp2) / s / exp1 / exp2 * zt2;
    n1 = (int)(n0 * s + 0.99);
    int n2 = (int)(n0 + 0.99);
    nd = (tmax > 0.0) ? (int)(s * n0 * exp1 + n0 * exp2 + 0.99) : n1 + n2;
    cout << "  n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << "  total deathes=" << nd << ")\n\n";
    cout << "Parametric Hazard Difference Test(between-group comparison)\n";
    cout << "  lambda1=" << lamda1 << "  lambda2=" << lamda2 << "\n";
    cout << "  HD(lambda1 - lambda2)=" << hd << "  t=";
    if (tmax > 0.0) cout << tmax << "\n";
    else cout << "infinity\n";
    double lamdam = (s * lamda1 + lamda2) / (1.0 + s);
    double w1 = lamda1 * lamda1, w2 = lamda2 * lamda2, wm = lamdam * lamdam;
    if (tmax > 0.0)
    {
      w1 /= exp1;  w2 /= exp2;
      wm /= (1.0 - exp(-lamdam * tmax));
    }
    zt = (za * sqrt((1.0 + s) * wm) + z2b * sqrt(s * w1 + w2)) / hd;
    n0 = zt * zt / s;
    n1 = (int)(n0 * s + 0.99);
    n2 = (int)(n0 + 0.99);
    cout << "  n1=" << n1 << "  n2=" << n2 << "  (total n=" << n1 + n2 << ")\n\n";
  }
}

void designclass::n_mci() const // 計量値の信頼区間の必要例数計算
{
  cout << "--- Confidence Interval of Metric Data ---\n";
  double cc = CCdef, e = 1.0, sd = 1.0, po = 0.0, d = 0.0;
  int nt = 0;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    sd = get_fdata("population SD(sigma)", DBL_MIN, DBL_MAX, sd);
    e = get_fdata("absolute precision(e)", DBL_MIN, DBL_MAX, e);
    po = get_fdata("power(1 - beta,0:not taken into account)", 0.0, Pmax, po);
    if (po > 0.0) d = get_fdata("bias(0:no bias)", 0.0, e - DBL_EPSILON, d);
    nt = get_fdata("n in the population(0:infinity)", 0.0, (double)INT_MAX, nt);
    double a = 1.0 - cc;
    double za = ptoz(a), de = e - d;
    double n0 = za * za * sd * sd / de / de;
    if (nt > 0) n0 = (double)nt / ((double)(nt - 1) / n0 + 1.0); // 母集団が無限大ではない時
    else n0 = (int)(n0 + 1.9);
    int n = (int)n0, df = (int)n0 - 1;
    double po0 = 0.0, t0 = ptot(a, df), se = sd / sqrt(n0);
    if (po > 0.0)
    {
      double b = (d == 0.0) ? 1.0 - po : 1.0 - po * de / e;
      double zab = za + ptoz(b);
      n0 = zab * zab * sd * sd / de / de;
      if (nt > 0) n0 = (double)nt / ((double)(nt - 1) / n0 + 1.0);
      else n0 = (int)(n0 + 0.9);
      n = (int)n0;
      df = n - 1;  t0 = ptot(a, df);  se = sd / sqrt(n0);
      double tl = (- e - d) / se + t0, tu = (e - d) / se - t0;
      po0 = ncttolp(tu, df) - ncttolp(tl, df);
      while (po0 < po)
      {
        n++;
        df = n - 1;
        t0 = ptot(a, df);  se = sd / sqrt((double)n);
        tl = (-e - d) / se + t0;  tu = (e - d) / se - t0;
        po0 = ncttolp(tu, df) - ncttolp(tl, df);
      }
    }
    cout << "Confidence Coefficient=" << cc << "  sigma=" << sd << "  e=" << e;
    if (nt > 0) cout << "  n_popu=" << nt;
    if (po > 0.0)
    {
      cout << "  Bias=" << d << "  power=" << po;
      cout << "\n  n=" << n << "  Actual Power=" << po0;
    }
    else
    { cout << "\n  n=" << n; }
    cout << "  Actual e=" << t0 * se << "\n\n";
  }
}

void designclass::n_rci() const // 計数値の信頼区間の必要例数計算
{
  cout << "--- Confidence Interval of Nominal Data ---\n";
  double cc = CCdef, p0 = 0.5, pci = 0.25;
  int nt = 0;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    p0 = get_fdata("proportion of the population(pi)", 0.0, 1.0, p0);
    pci = get_fdata("absolute precision(e)", DBL_EPSILON, 1.0, pci);
    nt = get_fdata("n in the population(0:infinity)", 0.0, (double)INT_MAX, nt);
    double za = ptoz(1.0 - cc);
    double n0 = za * za * p0 * (1.0 - p0) / pci / pci;
    cout << "Confidence Coefficient=" << cc << "  pi=" << p0 << "  e=" << pci;
    if (nt > 0)                 // 母集団が無限大ではない時
    {
      n0 = (double)nt / ((double)(nt - 1) / n0 + 1.0);
      cout << "  n_popu=" << nt;
    }
    int nt = (int)(n0 + 0.99);
    int n = (int)((n0 + 0.99) * p0 + 0.5);
    double plow = 0.0, pup = 1.0;
    calbexci(plow, pup, n, nt, cc);
    cout << "\n  n=" << nt << "  Exact CI:lower=" << plow << "  upper=" << pup << "\n\n";
  }
}

void designclass::n_icc() const // 級内相関係数の信頼区間の必要例数計算
{
  cout << "--- Confidence Interval of Intraclass Correlation Coefficient ---\n";
  constexpr int Nmax = 10000;
  double cc = CCdef, icc = 0.95, iccl = 0.9, b = 2.0;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    b = get_fdata("number of raters(b)", 2.0, (double)INT_MAX, b);
    icc = get_fdata("ICC(3,1)", 0.0 + DBL_EPSILON, 1.0 - DBL_EPSILON, icc);
    iccl = get_fdata("lower limit of ICC(3,1)", 0.0 + DBL_EPSILON, icc - DBL_EPSILON, iccl);
    double ah = (1.0 - cc) * 0.5;
    double fo = (1.0 + (b - 1.0) * icc) / (1.0 - icc);
    double fl = (1.0 + (b - 1.0) * iccl) / (1.0 - iccl);
    double fol = fo / fl;
    int n = 2, b1 = (int)b - 1;
    int df1 = n - 1, df2 = (n - 1) * ((int)b - 1);
    double f = ptof(ah, df1, df1 * b1);
    while (f > fol && n < Nmax)
    {
      df1 = (++n) - 1;
      f = ptof(ah, df1, df1 * b1);
    }
    fl = fo / f;
    iccl = (fl - 1.0) / (fl + b - 1.0);
    double fu = fo / ptof(1.0 - ah, df1, df1 * b1);
    double iccu = (fu - 1.0) / (fu + b - 1.0);
    cout << "Confidence Coefficient=" << cc << "  raters=" << b << "  ICC(3,1)=" << icc << "\n";
    cout << "  ICC(3,1):lower=" << iccl << "  upper=" << iccu << "  n=" << n << "\n\n";
  }
}

void designclass::n_kappa() const // 一致係数κの信頼区間の必要例数計算
{
  cout << "--- CI of Matching Coefficient Kappa ---\n";
  constexpr int Cmax = 1000;
  double cc = CCdef, kappa = 0.8, e = 0.1, c = 2.0;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    c = get_fdata("number of categories(c)", 2.0, (double)Cmax, c);
    kappa = get_fdata("kappa", 0.0 + DBL_EPSILON, 1.0 - DBL_EPSILON, kappa);
    e = get_fdata("absolute precision(e)", DBL_MIN, 1.0, e);
    double a = 1.0 - cc;
    double za = ptoz(a);
    double pc = 1.0 / c;
    double p0 = kappa * (1.0 - pc) + pc, pc1 = 1.0 - pc;
    double n0 = za * za * p0 * (1.0 - p0) / e / e / pc1 / pc1;
    int n = (int)(n0 + 0.99);
    cout << "Confidence Coefficient=" << cc << "  kappa=" << kappa << "  e=" << e;
    cout << "  n=" << n << "\n\n";
  }
}

void designclass::n_hazard() const // ハザードλの信頼区間の必要例数計算
{
  cout << "--- Confidence Interval of Hazard ---\n";
  double cc = CCdef, lam = 0.1, e = 0.05, t = 0.0;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    lam = get_fdata("hazrad of the population(lambda)", 0.0, DBL_MAX, lam);
    e = get_fdata("absolute precision(e)", DBL_EPSILON, lam, e);
    t = get_fdata("observation period(t,0:infinity)", 0.0, DBL_MAX, t);
    double a = 1.0 - cc;
    double za = ptoz(a);
    double ex = (t > 0.0) ? 1.0 - exp(-lam * t) : 1.0;
    double n0 = za * za * lam * lam / e / e / ex;
    cout << "Confidece Coefficient=" << cc << "  lambda=" << lam << "  e=" << e << "  t=";
    if (t > 0.0) cout << t;
    else cout << "infinity";
    cout << "\n  Mean Survival Time:1/lambda=" << 1.0 / lam << "  lower=" << 1.0 / (lam + e) << "  upper=" << 1.0 / (lam - e);
    int n = (int)(n0 + 0.99);
    cout << "  n=" << n << "\n\n";
  }
}

void designclass::n_lracl() const // 回帰直線の信頼限界の必要例数計算
{
  cout << "--- Confidence Interval of Regression Line ---\n";
  double cc = CCdef, e = 1.0, vr = 1.0;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    vr = get_fdata("residuals variance(VR)", DBL_MIN, DBL_MAX, vr);
    e = get_fdata("absolute precision(e)", DBL_MIN, DBL_MAX, e);
    double a = 1.0 - cc;
    double za = ptoz(a);
    double n0 = za * za * vr / e / e;
    n0 = (int)(n0 + 2.9);
    int n = (int)n0, df = (int)n0 - 2;
    double t0 = ptot(a, df), se = sqrt(vr / n0);
    cout << "Confidence Coefficient=" << cc << "  VR=" << vr << "  e=" << e;
    cout << "\n  When x=mean:n=" << n << "  When x=mean +/- SD:n=" << 2 * n;
    cout << "  Actual e=" << t0 * se << "\n\n";
  }
}

void designclass::po_ttest() const // 計量値(t検定)の検出力計算
{
  cout << "--- Metric Data(t-test) ---\n";
  double a = ALdef, n1 = 2.0, m1 = 1.0, sd1 = 1.0, n2 = 2.0, m2 = 0.0, sd2 = 1.0;
  while ((a = get_fdata("siginificance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    n1 = get_fdata("n of group1(n1)", 2.0, (double)INT_MAX, n1);
    m1 = get_fdata("mean of group1(m1)", -DBL_MAX, DBL_MAX, m1);
    sd1 = get_fdata("SD of group1(SD1)", DBL_MIN, DBL_MAX, sd1);
    n2 = get_fdata("n of group2(n2,0:within-group comparison)", 0.0, (double)INT_MAX, n2);
    int df = (int)n1 - 1;
    double dm = fabs(m1), sd = sd1, ta = 0.0, t2b = 0.0;
    double d = dm / sd;         // 効果量
    double lambda = d * sqrt(n1); // 非心度
    int nt = (int)(n1 + n2);
    if (n2 >= 1.0)              // 2群比較
    {
      m2 = get_fdata("mean of group2(m2)", -DBL_MAX, DBL_MAX, m2);
      sd2 = get_fdata("SD of group2(SD2)", DBL_MIN, DBL_MAX, sd2);
      cout << "Comparison of 2 groups(between-group comparison)\n";
      cout << "  n1=" << n1 << "  SD1=" << sd1 << "  n2=" << n2 << "  SD2=" << sd2 << "\n";
      df = nt - 2;
      dm = fabs(m1 - m2);
      sd = sqrt((sd1 * sd1 * (n1 - 1.0) + sd2 * sd2 * (n2 - 1.0)) / (double)df);
      d = dm / sd;
      lambda = d / sqrt(1.0 / n1 + 1.0 / n2);
      ta = ptot(a, df);
      t2b = lambda - ta;
      double r2 = d * d / (d * d + (double)(nt - 2) * (double)nt / n1 / n2);
      cout << "  Correlation Ratio:r=" << sqrt(r2) << "  r-squared=" << r2 << "\n";
    }
    else
    {
      cout << "Comparison of 1 group(within-group comparison)\n";
      cout << "  n1=" << n1 << "  SD1=" << sd1 << "\n";
      ta = ptot(a, df);
      t2b = lambda - ta;
    }
    double b = (t2b > 0.0) ? ttop(t2b, df) * 0.5 : 1.0 - ttop(-t2b, df) * 0.5;
    double b0 = ncttolp(ta, df, lambda);
    cout << "  Significant Difference:d=" << dm << "  common SD=" << sd << "\n";
    cout << "  Effect size:d/SD=" << d << "  Noncentral Degree=" << lambda << "\n";
    cout << "  Power(normal approximation)=" << 1.0 - b << "(beta=" << b << ")\n";
    cout << "  Power(noncentral t distribution)=" << 1.0 - b0 << "(beta=" << b0 << ")\n\n";
  }
}

void designclass::po_tmcomp() const // 計量値(多重比較)の検出力計算
{
  cout << "--- Metric Data(Multiple Comparison) ---\n";
  int g = 0, gold = 0;
  double a = ALdef;
  statv<double> dummys;
  dummys.set(10, 0.0, 1.0);
  vector<statv<double>> statvec;
  while ((g = (int)(get_fdata("number of groups(0:quit)", 0.0, 100.0, (double)g))) > 1)
  {
    a = get_fdata("siginificance level(alpha)", Pmin, Pmax, a);
    if (statvec.size() > 0 && gold != g) statvec.clear();
    if (statvec.empty()) statvec.resize(g, dummys);
    int nt = 0;
    double sst = 0.0;
    for (int i = 0; i < g; ++i)
    {
      ostringstream ostr;
      ostr << "n of group" << i + 1 << "(n" << i + 1 << ")";
      int n = (int)(get_fdata(ostr.str(), 2.0, (double)INT_MAX, (double)(statvec[i].n())));
      ostr.str("");
      ostr << "mean of group" << i + 1 << "(m" << i + 1 << ")";
      double mean = get_fdata(ostr.str(), -DBL_MAX, DBL_MAX, statvec[i].mean());
      ostr.str("");
      ostr << "SD of group" << i + 1 << "(SD" << i + 1 << ")";
      double sd = get_fdata(ostr.str(), DBL_MIN, DBL_MAX, statvec[i].sd());
      statvec[i].set(n, mean, sd);
      nt += n;
      sst += statvec[i].ss();
    }
    int df = nt - g;
    sst = sqrt(sst / (double)df);
    cout << "Fisher type:alpha=" << a << "\n";
    put_tpower(statvec, sst, ptot(a, df), df, false);
    int nc = g * (g - 1) / 2;
    double ac = a / (double)nc; // 多重性調整済αエラー
    double tac = ptot(ac, df);
    cout << "Bonferroni type:alpha_c=" << ac << "\n";
    put_tpower(statvec, sst, tac, df);
    cout << "Scheffe type:alpha=" << a << "\n";
    int df1 = g - 1;
    tac = sqrt(ptof(a, df1, df) * df1);
    put_tpower(statvec, sst, tac, df);
    put_tpowersim(statvec, a);
    gold = g;
  }
}

void designclass::po_ctest(const option& opt) const
// 計数値(χ2乗検定)の検出力計算
{
  cout << "--- Nominal Data(Chi-squared Test) ---\n";
  double n1 = 2.0, n2 = 2.0, a = ALdef, pi1 = 0.7, pi2 = 0.5;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    n1 = get_fdata("n of group1(n1)", 2.0, (double)INT_MAX, n1);
    pi1 = get_fdata("proportion of group1(pi1)", 0.0, 1.0, pi1);
    n2 = get_fdata("n of group2(n2,0:within-group comparison)", 0.0, (double)INT_MAX, n2);
    if (n2 >= 1.0)              // 2群比較
    {
      while ((pi2 = get_fdata("proportion of group2(pi2)", 0.0, 1.0, pi2)) == pi1) disperr(opt.pname(), "can not compute when pi1=pi2 !");
      double dpi = fabs(pi1 - pi2);
      double s = n2 / n1;
      double pi = (pi1 + s * pi2) / (s + 1.0);
      double v = pi * (1.0 - pi);
      double za = ptoz(a);
      double tmp1 = za * sqrt((s + 1.0) * v);
      double tmp2 = sqrt(s * pi1 * (1.0 - pi1) + pi2 * (1.0 - pi2));
      double z2b = (sqrt(s * dpi * dpi * n1) - tmp1) / tmp2;
      double po = nedp(z2b);
      double r = sqrt(s) / (1.0 + s) * dpi / sqrt(v);
      cout << "Comparison of 2 groups(between-group comparison)\n";
      cout << "  n1=" << n1 << "  pi1=" << pi1 << "  n2=" << n2 << "  pi2=" << pi2 << "\n";
      cout << "  Coefficient of Contingency:V=" << r << "  V-squared=" << r * r << "\n";
      cout << "  No continuous correction:Power=" << po << "(beta=" << 1.0 - po << ")\n";
      double n1t = n1 * 0.9;
      double tmp3 = 1.0 + sqrt(1.0 + 2.0 * (s + 1.0) / s / n1t / dpi);
      n1t = 4.0 * n1 / tmp3 / tmp3;
      z2b = (sqrt(s * dpi * dpi * n1t) - tmp1) / tmp2;
      po = nedp(z2b);
      cout << "  With continuous correction:Power=" << po << "(beta=" << 1.0 - po << ")\n\n";
    }
    else                        // 1群比較
    {
      while ((pi2 = get_fdata("theoretical proportion(pi0)", 0.0, 1.0, pi2)) == pi1) disperr(opt.pname(), "can not compute when pi1=pi0 !");
      double dpi = fabs(pi1 - pi2);
      double za = ptoz(a);
      double z2b = (dpi * sqrt(n1) - za * sqrt(pi2 * (1.0 - pi2))) / sqrt(pi1 * (1.0 - pi1));
      double po = nedp(z2b);
      cout << "Compariosn of 1 group(within-group comparison)\n";
      cout << "  n1=" << n1 << "  pi1=" << pi1 << "  pi0=" << pi2 << "\n";
      cout << "  Power=" << po << "(beta=" << 1.0 - po << ")\n\n";
    }
  }
}

void designclass::po_cmcomp() const // 計数値(多重比較)の検出力計算
{
  cout << "--- Nominal Data(Multiple Comparison) ---\n";
  int g = 0, gold = 0;
  double a = ALdef;
  statv<double> dummys(10, 5.0, 0.0);
  vector<statv<double>> statvec;
  while ((g = (int)(get_fdata("number of groups(0:quit)", 0.0, 100.0, (double)g))) > 1)
  {
    a = get_fdata("significance level(alpha)", Pmin, Pmax, a);
    if (statvec.size() > 0 && gold != g) statvec.clear();
    if (statvec.empty()) statvec.resize(g, dummys);
    int nt = 0;
    for (int i = 0; i < g; ++i)
    {
      ostringstream ostr;
      ostr << "n of group" << i + 1 << "(n" << i + 1 << ")";
      int n = (int)(get_fdata(ostr.str(), 2.0, (double)INT_MAX, (double)(statvec[i].n())));
      ostr.str("");
      ostr << "proportion of group" << i + 1 << "(pi" << i + 1 << ")";
      double mean = get_fdata(ostr.str(), 0.0, 1.0, statvec[i].mean());
      statvec[i].set(n, mean, 0.0);
      nt += n;
    }
    cout << "Fisher type:alpha=" << a << "\n";
    put_cpower(statvec, ptoz(a), false);
    int nc = g * (g - 1) / 2;
    double ac = a / (double)nc; // 多重性調整済αエラー
    double zac = ptoz(ac);
    cout << "Bonferroni type:alpha_c=" << ac << "\n";
    put_cpower(statvec, zac);
    cout << "Scheffe type:alpha=" << a << "\n";
    zac = sqrt(ptoc(a, g - 1));
    put_cpower(statvec, zac);
    put_cpowersim(statvec, a);
    gold = g;
  }
}

void designclass::po_life(const option& opt) const // 生存時間解析の検出力計算
{
  cout << "--- Survival Time Analysis(PHR-test) ---\n";
  double n1 = 2.0, n2 = 2.0, nd1 = 1.0, nd2 = 1.0;
  double a = ALdef, lamda1 = 0.1, lamda2 = 0.2;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    n1 = get_fdata("n of group1(n1)", 2.0, (double)INT_MAX, n1);
    nd1 = get_fdata("deaths of group1(deaths1)", 1.0, (double)INT_MAX, nd1);
    lamda1 = get_fdata("hazard of group1(lambda1)", FDmin, 1.0, lamda1);
    n2 = get_fdata("n of group2(n2)", 2.0, (double)INT_MAX, n2);
    nd2 = get_fdata("deaths of group2(deaths2)", 1.0, (double)INT_MAX, nd2);
    while ((lamda2 = get_fdata("hazard of group2(lambda2)", Pmin, 1.0, lamda2)) == lamda1) disperr(opt.pname(), "can not compute when lambda1=lambda2 !");
    {
      double hr = lamda2 / lamda1, za = ptoz(a);
      double b1 = log(hr), seb1 = sqrt(1.0 / nd1 + 1.0 / nd2);
      double z2b = fabs(b1) / seb1 - za;
      double po = nedp(z2b);
      cout << "Parametric Hazard Ratio Test(2 groups comparison)\n";
      cout << "  n1=" << n1 << "  deaths1=" << nd1 << "  lambda1=" << lamda1 << "\n";
      cout << "  n2=" << n2 << "  deaths2=" << nd2 << "  labmda2=" << lamda2 << "\n";
      cout << "  Hazard Ratio:HR(lambda2/lambda1)=" << hr << "  ln(HR)=" << b1 << "\n";
      cout << "  Power=" << po << "(beta=" << 1.0 - po << ")\n\n";
    }
  }
}

void designclass::put_tpower(vector<statv<double>>& statvec, double sd, double ta, int df, bool bcput) const
// 計量値(多重比較)の検出力出力
{
  int g = statvec.size();
  double bc = 1.0, bc0 = 1.0;
  for (int i = 0; i < g - 1; ++i)
  {
    for (int j = i + 1; j < g; ++j)
    {
      double d = fabs(statvec[i].mean() - statvec[j].mean());
      double lambda = d / sd / sqrt(1.0 / (double)(statvec[i].n()) + 1.0 / (double)(statvec[j].n()));
      double t2b = lambda - ta;
      double b = (t2b > 0.0) ? ttop(t2b, df) * 0.5 : 1.0 - ttop(-t2b, df) * 0.5;
      double b0 = ncttolp(ta, df, lambda);
      cout << "  m" << i + 1 << " - m" << j + 1 << ":" << "Significant Difference=" << d << "  common SD=" << sd << "  Noncentral Degree=" << lambda << "\n";
      cout << "  Power(normal approximation)=" << 1.0 - b << "(beta=" << b << ")\n";
      cout << "  Power(noncentral t-distribution)=" << 1.0 - b0 << "(beta=" << b0 << ")\n";
      bc *= b;  bc0 *= b0;
    }
  }
  if (bcput && g > 2)
  {
    cout << "  Overall Power(normal approximation)=" << 1.0 - bc << "(beta=" << bc << ")\n";
    cout << "  Overall Power(non-cantral t-distribution)=" << 1.0 - bc0 << "(beta=" << bc0 << ")\n";
  }
  cout << "\n";
}

void designclass::put_cpower(vector<statv<double>>& statvec, double za, bool bcput) const
// 計数値(多重比較)の検出力出力
{
  int g = statvec.size();
  double bc = 1.0, bcc = 1.0;
  for (int i = 0; i < g - 1; ++i)
  {
    for (int j = i + 1; j < g; ++j)
    {
      double pi1 = statvec[i].mean(), pi2 = statvec[j].mean();
      if (sgndiff(pi1, pi2) == 0)
      {
        cout << "  pi" << i + 1 << " - pi" << j + 1 << ":can not compute when 2 proportion are equal !\n";
      }
      else
      {
        double dpi = fabs(pi1 - pi2);
        double n1 = statvec[i].n();
        double r = (double)(statvec[j].n()) / n1;
        double pi = (pi1 + r * pi2) / (r + 1.0);
        double v = pi * (1.0 - pi);
        double tmp1 = za * sqrt((r + 1.0) * v);
        double tmp2 = sqrt(r * pi1 * (1.0 - pi1) + pi2 * (1.0 - pi2));
        double z2b = (sqrt(r * dpi * dpi * n1) - tmp1) / tmp2;
        double po = nedp(z2b);
        double b = 1.0 - po;
        cout << "  pi" << i + 1 << " - pi" << j + 1 << ":Power=" << po << "(beta=" << b << ")\n";
        bc *= b;
        double n1t = n1 * 0.9;
        double tmp3 = 1.0 + sqrt(1.0 + 2.0 * (r + 1.0) / r / n1t / dpi);
        n1t = 4.0 * n1 / tmp3 / tmp3;
        z2b = (sqrt(r * dpi * dpi * n1t) - tmp1) / tmp2;
        po = nedp(z2b);
        b = 1.0 - po;
        cout << "  Power with correction=" << po << "(beta=" << b << ")\n";
        bcc *= b;
      }
    }
  }
  if (bcput && g > 2)
  {
    cout << "  Overall Power=" << 1.0 - bc << "(beta=" << bc << ")\n";
    cout << "  Power with correction=" << 1.0 - bcc << "(beta=" << bcc << ")\n";
  }
  cout << endl;
}

void designclass::put_tpowersim(vector<statv<double>>& statvec, double a) const
// シミュレーションによる計量値(多重比較)の検出力出力
{
  int g = statvec.size();
  vector<statv<double>> stativec(g + 1); // 1回1回の基礎統計量
  int nasig = 0, nfsig = 0, nbsig = 0, nssig = 0; // 有意になった回数
  int nc = g * (g - 1) / 2;     // 多重比較の比較回数
  srand(time((long *)0));       // 乱数のタネをまく
  constexpr int Rdef = 10000;   // 繰り返し回数
  for (int i = 0; i < Rdef; ++i)
  {
    for (int j = 0; j < g + 1; ++j) stativec[j].clear();
    for (int j = 0; j < g; ++j)
    {
      int nj = statvec[j].n();
      double mj = statvec[j].mean(), sdj = statvec[j].sd();
      for (int k = 0; k < nj; ++k) stativec[j].sigma(randn(mj, sdj));
    }
    double sa = 0.0;
    for (int j = 0; j < g; ++j)
    {
      stativec[g] += stativec[j];
      int nj = stativec[j].n();
      double mj = stativec[j].mean();
      sa += nj * mj * mj;
    }
    double st = stativec[g].ss();
    int nt = stativec[g].n();
    double mt = stativec[g].mean();
    sa -= nt * mt * mt;
    double sr = st - sa;
    int dft = nt - 1, dfa = g - 1, dfr = nt - g;
    const double Seps = DBL_EPSILON * fmax(FDmin, st);
    if (sa < Seps || sr < Seps || dfr < 1) continue; // 検定できず
    double vr = sr / dfr;
    double f = (sa / dfa) / vr;
    double pr = ftop(f, dfa, dfr);
    if (pr < a) ++nasig;
    bool fsig = false, bsig = false, ssig = false;
    for (int j = 0; (bsig == false || ssig == false) && j < g - 1; ++j)
    {
      int nj = stativec[j].n();
      double mj = stativec[j].mean();
      for (int k = j + 1; (bsig == false || ssig == false) && k < g; ++k)
      {
        int nk = stativec[k].n();
        double mk = stativec[k].mean();
        double dm = mj - mk;
        f = dm * dm / (1.0 / (double)nj + 1.0 / (double)nk) / vr;
        pr = ftop(f, 1, dfr);
        if (j == 0 && k == 1)   // Fisher型は1度だけ検定
        {
          if (pr < a) fsig = true;
        }
        pr *= nc;
        if (pr < a) bsig = true;
        f = dm * dm / dfa / (1.0 / (double)nj + 1.0 / (double)nk) / vr;
        pr = ftop(f, dfa, dfr);
        if (pr < a) ssig = true;
      }
    }
    if (fsig) ++nfsig;
    if (bsig) ++nbsig;
    if (ssig) ++nssig;
  }
  double po = (double)nasig / (double)Rdef;
  cout << "Power by Simulation(repetitions:" << Rdef << ")\n";
  cout << "  Power of ANOVA1=" << po << "(beta=" << 1.0 - po << ")\n";
  po = (double)nfsig / (double)Rdef;
  cout << "  Power of Fisher Type(m1 - m2)=" << po << "(beta=" << 1.0 - po << ")\n";
  po = (double)nbsig / (double)Rdef;
  cout << "  Overall Power of Bonferroni Type=" << po << "(beta=" << 1.0 - po << ")\n";
  po = (double)nssig / (double)Rdef;
  cout << "  Overall Power of Scheffe Type=" << po << "(beta=" << 1.0 - po << ")\n\n";
}

void designclass::put_cpowersim(vector<statv<double>>& statvec, double a) const
// シミュレーションによる計数値(多重比較)の検出力出力
{
  int g = statvec.size();
  ftable ftabi(g, 2);           // 1回1回の度数分布表
  int nasig = 0, nfsig = 0, nbsig = 0, nssig = 0; // 有意になった回数
  int nc = g * (g - 1) / 2;     // 多重比較の比較回数
  srand(time((long *)0));       // 乱数のタネをまく
  constexpr int Rdef = 10000;   // 繰り返し回数
  for (int i = 0; i < Rdef; ++i)
  {
    ftabi.fclear();
    for (int j = 0; j < g; ++j)
    {
      int nj = statvec[j].n();
      double pij = statvec[j].mean();
      double mj = (double)nj * pij, sdj = sqrt((double)nj * pij * (1.0 - pij));
      int np = (int)(randn(mj, sdj)); // 正規乱数を利用して度数を設定する
      ftabi.assign(j + 1, 1, np);  ftabi.assign(j + 1, 2, nj - np);
    }
    ftabi.set();
    double csqr = 0.0;
    int df = 0;
    ctest(csqr, df, ftabi, false);
    if (sgn(csqr) <= 0) continue; // 検定できず
    double pr = ctop(csqr, df);
    if (pr < a) ++nasig;
    int nt = ftabi.ftotal(), dff = ftabi.nc() - 1;
    bool fsig = false, bsig = false, ssig = false;
    for (int j = 1; (bsig == false || ssig == false) && j < g; ++j)
    {
      double xj = (double)(ftabi.frtotal(j));
      for (int k = j + 1; (bsig == false || ssig == false) && k <= g; ++k)
      {
        double xk = (double)(ftabi.frtotal(k));
        csqr = 0.0;
        for (int l = 1; l <= (int)(ftabi.nc()); ++l)
        {
          double pd = (double)(ftabi.f(j, l)) / xj - (double)(ftabi.f(k, l)) / xk;
          csqr += pd * pd / (double)(ftabi.fctotal(l));
        }
        csqr *= nt * xj * xk / (xj + xk);
        pr = ctop(csqr, dff);
        if (j == 1 && k == 2)   // Fisher型は1度だけ検定
        {
          if (pr < a) fsig = true;
        }
        pr *= nc;
        if (pr < a) bsig = true;
        pr = ctop(csqr, df);
        if (pr < a) ssig = true;
      }
    }
    if (fsig) ++nfsig;
    if (bsig) ++nbsig;
    if (ssig) ++nssig;
  }
  double po = (double)nasig / (double)Rdef;
  cout << "Power by Simulation(repetitions:" << Rdef << ")\n";
  cout << "  Power of Chi-squared Test(" << g << "×2)=" << po << "(beta=" << 1.0 - po << ")\n";
  po = (double)nfsig / (double)Rdef;
  cout << "  Power of Fisher Type(pi1 - pi2)=" << po << "(beta=" << 1.0 - po << ")\n";
  po = (double)nbsig / (double)Rdef;
  cout << "  Overall Power of Bonferroni Type=" << po << "(beta=" << 1.0 - po << ")\n";
  po = (double)nssig / (double)Rdef;
  cout << "  Overall Power of Scheffe Type=" << po << "(beta=" << 1.0 - po << ")\n\n";
}

void designclass::diff_ttest() const // 計量値(t検定)の検出差計算
{
  cout << "--- Metric Data(t-test) ---\n";
  double a = ALdef, po = POdef, n1 = 2.0, sd1 = 1.0, n2 = 2.0, sd2 = 1.0;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    n1 = get_fdata("n of group1(n1)", 2.0, (double)INT_MAX, n1);
    sd1 = get_fdata("SD of group1(SD1)", DBL_MIN, DBL_MAX, sd1);
    int df = (int)n1 - 1;
    double sd = sd1, ta = 0.0, t2b = 0.0, lamda = 0.0;
    n2 = get_fdata("n of group2(n2,0:within-group comparison)", 0.0, (double)INT_MAX, n2);
    int nt = (int)(n1 + n2);
    if (n2 >= 1.0)              // 2群比較
    {
      sd2 = get_fdata("SD if group2(SD2)", DBL_MIN, DBL_MAX, sd2);
      cout << "Comparison of 2 groups(between-group comparison)\n";
      cout << "  n1=" << n1 << "  SD1=" << sd1 << "  n2=" << n2 << "  SD2=" << sd2 << "\n";
      df = nt - 2;
      sd = sqrt((sd1 * sd1 * (n1 - 1.0) + sd2 * sd2 * (n2 - 1.0)) / (double)df);
      double s = n2 / n1;
      ta = ptot(a, df);  t2b = ptot(2.0 * (1.0 - po), df);
      double nc = ta * ta / 4.0 / (1.0 - 1.0 / (double)nt);
      if (n1 <= nc) nc = 0.0;
      lamda = (ta + t2b) * sqrt((1.0 + s) / s / (n1 - nc));
      double r2 = lamda * lamda / (lamda * lamda + (double)(nt - 2) * (double)nt / n1 / n2);
      cout << "  Correlation Ratio:r=" << sqrt(r2) << "  r-squared=" << r2 << "\n";
    }
    else
    {
      cout << "Comparison of 1 group(within-group comparison)\n";
      cout << "  n1=" << n1 << "  SD1=" << sd1 << "\n";
      ta = ptot(a, df), t2b = ptot(2.0 * (1.0 - po), df);
      double nc = (n1 > 1.0) ? ta * ta / 2.0 / (1.0 - 1.0 / n1) : 0.0;
      if (n1 <= nc) nc = 0.0;
      lamda = (ta + t2b) / sqrt(n1 - nc);
    }
    double d = lamda * sd;
    cout << "  Significant Difference:d=" << d << "  common SD=" << sd << "  Effect Size:d/SD=" << lamda << "\n\n";
  }
}

void designclass::diff_ctest() const // 計数値(χ2乗検定)の検出差計算
{
  cout << "--- Nominal Data(Chi-squared Test) ---\n";
  double n1 = 2.0, n2 = 2.0, a = ALdef, po = POdef, pi = 0.5;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    n1 = get_fdata("n of group1(n1)", 2.0, (double)INT_MAX, n1);
    n2 = get_fdata("n of group2(n2,0:within-group comparison)", 0.0, (double)INT_MAX, n2);
    if (n2 >= 1.0)              // 2群比較
    {
      pi = get_fdata("overall proportion(pi0)", 0.0, 1.0, pi);
      double s = n2 / n1;
      double v = pi * (1.0 - pi);
      double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
      double dpi = (za + z2b) * sqrt((1.0 + s) / s * v / n1);
      double pi1 = pi - dpi * s / (1.0 + s);
      double pi2 = pi1 + dpi;
      double r = sqrt(s) / (1.0 + s) * dpi / sqrt(v);
      cout << "Comparison of 2 groups(between-group comparison)\n";
      cout << "  No continuous correction:Significant Difference=" << dpi << "\n";
      cout << "  Coefficient of Contingency:V=" << r << "  V-squared=" << r * r << "\n";
      cout << "  Overall Proportion:pi0=" << pi << "\n";
      cout << "  n1=" << n1 << "  pi1=" << pi1 << "  n2=" << n2 << "  pi2=" << pi2 << "\n";
      cout << "  pi2/pi1=" << pi2 / pi1 << "\n\n";
    }
    else                        // 1群比較
    {
      double za = ptoz(a), z2b = ptoz(2.0 * (1.0 - po));
      double dpi = (za + z2b) / sqrt(n1);
      double pi1 = 0.5 + dpi / 2.0, pi2 = 0.5 - dpi / 2.0;
      cout << "Comparison of 1 group(within-group comparison)\n";
      cout << "  No continuous correction:Significant Difference=" << dpi << "\n";
      cout << "  Theoretical Proportion of yes=0.5\n";
      cout << "  n1=" << n1 << "  Proportion of yes:pi+=" << pi1 << "  Proportion of no:pi-=" << pi2 << "\n";
      cout << "  pi+/pi-=" << pi1 / pi2 << "\n\n";
    }
  }
}

void designclass::diff_life() const // 生存時間解析の検出差計算
{
  cout << "--- Survival Time Analysis(PHR-test) ---\n";
  double nd1 = 1.0, nd2 = 1.0, a = ALdef, po = POdef;
  while ((a = get_fdata("significance level(alpha,0:quit)", 0.0, Pmax, a)) > 0.0)
  {
    po = get_fdata("power(1 - beta)", Pmin, Pmax, po);
    nd1 = get_fdata("deaths of group1(deaths1)", 1.0, (double)INT_MAX, nd1);
    nd2 = get_fdata("deaths of group2(deaths2)", 1.0, (double)INT_MAX, nd2);
    double beta = 1.0 - po;
    double za = ptoz(a), z2b = ptoz(beta * 2.0);
    double seb1 = sqrt(1.0 / nd1 + 1.0 / nd2);
    double b1 = (za + z2b) * seb1;
    cout << "Parametric Hazard Ratio Test(2 group comparison)\n";
    cout << "  deaths1=" << nd1 << "  deaths2=" << nd2 << "\n";
    cout << "  Detecting Hazard Ratio:HR(lambda2/lambda1)=" << exp(b1) << "  ln(HR)=" << b1 << "\n\n";
  }
}

void designclass::absp_ttest() const // 計量値の信頼区間の絶対精度計算
{
  cout << "--- Confidence Interval of Metric Data ---\n";
  double cc = CCdef, sd = 1.0;
  int n = 20;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    sd = get_fdata("population SD(sigma)", DBL_MIN, DBL_MAX, sd);
    n = get_fdata("n", 2.0, (double)INT_MAX, (double)n);
    double al = 1.0 - cc;
    int df = n - 1;
    double t0 = ptot(al, df);
    double e = t0 * sd / sqrt((double)n);
    cout << "Confidence Coefficient=" << cc << "  sigma=" << sd << "  n=" << n;
    cout << "  Absolute Precision=" << e << "\n\n";
  }
}

void designclass::absp_ctest() const // 計数値の信頼区間の絶対精度計算
{
  cout << "--- Confidence Interval of Nominal Data ---\n";
  double cc = CCdef, p0 = 0.5;
  int nt = 20;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    p0 = get_fdata("proportion of the population(pi)", DBL_EPSILON, 1.0, p0);
    nt = get_fdata("n", 2.0, (double)INT_MAX, (double)nt);
    double plow = 0.0, pup = 1.0;
    double pci = calbnmci(plow, pup, p0, nt, cc);
    cout << "Confidence Coefficient=" << cc << "  pi=" << p0 << "  n=" << nt << "  Absolute Precision=" << pci << "\n";
    int n = (int)((double)nt * p0 + 0.5);
    pci = calbexci(plow, pup, n, nt, cc);
    cout << "  Exact CI:lower=" << plow << "  upper=" << pup << "  Range of CI=" << pci << "\n\n";
  }
}

void designclass::absp_lra() const // 回帰直線の信頼限界の絶対精度計算
{
  cout << "--- Confidence Interval of Regression Line ---\n";
  double cc = CCdef, vr = 1.0;
  int n = 20;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    vr = get_fdata("residual variance(VR)", DBL_MIN, DBL_MAX, vr);
    n = get_fdata("n", 2.0, (double)INT_MAX, (double)n);
    double al = 1.0 - cc;
    int df = n - 2;
    double t0 = ptot(al, df);
    double e = t0 * sqrt(vr / (double)n);
    cout << "Confidence Coefficient=" << cc << "  VR=" << vr << "  n=" << n;
    cout << "\n  Absolute Precision:e(x=mean)=" << e << "  e(x=mean +/- SD)=" << e * sqrt(2.0) << "\n\n";
  }
}

void designclass::aepr_binom() const // 有害事象が発現する確率計算
{
  cout << "--- Probability of Adverse Events Occurring ---\n";
  double p0 = 0.1;
  int nt = 20;
  while ((p0 = get_fdata("proportion of the population(pi,0:quit)", 0.0, 1.0, p0)) > 0.0)
  {
    nt = get_fdata("n", 2.0, (double)INT_MAX, (double)nt);
    double pr = binom(0, nt, p0);
    cout << "pi=" << p0 << "  n=" << nt << "\n";
    cout << "  One-tailed p-value when AE=0:p=" << pr << "\n";
    cout << "  Probability of one or more AE occurring:p=" << 1.0 - pr << "\n\n";
  }
}

void designclass::aepr_pici() const // 母出現率と絶対精度計算
{
  cout << "--- Proportion and Absolute Precision ---\n";
  double cc = CCdef, rci = 1.0;
  int nt = 20;
  while ((cc = get_fdata("confidence coefficient(1 - alpha,0:quit)", 0.0, Pmax, cc)) > 0.0)
  {
    rci = get_fdata("precision／proportion ratio(PPR)", DBL_MIN, DBL_MAX, rci);
    nt = get_fdata("n", 2.0, (double)INT_MAX, (double)nt);
    double za = ptoz(1.0 - cc);
    double p0 = za * za / (rci * rci * (double)nt + za * za);
    if (p0 >= 1.0) p0 = 1.0 - DBL_EPSILON;
    double plow = 0.0, pup = 1.0;
    double pci = calbnmci(plow, pup, p0, nt, cc, false); // 連続補正なし
    cout << "Confidence Coefficient=" << cc << "  PPR=" << rci << "  n=" << nt << "\n";
    cout << "  Proportion=" << p0 << "  Absolute Precision=" << pci << "\n";
    cout << "  CI:lower=" << plow << "  upper=" << pup << "\n";
    int n = (int)((double)nt * p0 + 0.5);
    pci = calbexci(plow, pup, n, nt, cc);
    cout << "  Exact CI:lower=" << plow << "  upper=" << pup << "  Range of CI=" << pci << "\n\n";
  }
}

int designclass::sel(int kind) const // メニュー表示と選択
{
  puttitle(cout, Title);
  cout << "\n";
  cout << "1:Calculation of the Sample Size\n";
  cout << "2:Calculation of the Power(Beta Error)\n";
  cout << "3:Calculation of the Significant Difference\n";
  cout << "4:Calculation of the Absolute Precision of CI\n";
  cout << "5:Calculation of the Probability of AE\n\n";
  cout << "9:Show the Online Manual\n";
  cout << "0:Quit\n\n";
  return (int)(get_fdata("enter number", 0.0, 9.0, (double)kind));
}

void designclass::caln(const option& opt, int kind) const // 実験必要例数の計算
{
  int method = 0;
  while ((method = sel_method(opt, "Calculation of the Sample Size", kind, method)) != 0)
  {
    switch (method)
    {
      case 1 : n_ttest();
        break;
      case 2 : n_tmcomp();
        break;
      case 3 : n_ctest(opt);
        break;
      case 4 : n_cmcomp(opt);
        break;
      case 5 : n_mcnemar(opt);
        break;
      case 6 : n_cohort();
        break;
      case 7 : n_genom(opt);
        break;
      case 8 : n_w1test();
        break;
      case 9 : n_utest();
        break;
      case 10 : n_catest();
        break;
      case 11 : n_cor(opt);
        break;
      case 12 : n_life(opt);
        break;
      case 13 : n_mci();
        break;
      case 14 : n_rci();
        break;
      case 15 : n_icc();
        break;
      case 16 : n_kappa();
        break;
      case 17 : n_hazard();
        break;
      case 18 : n_lracl();
        break;
      default : disperr(opt.pname(), "Wrong number specified !");
    }
  }
}

void designclass::calpower(const option& opt, int kind) const // 検出力の計算
{
  int method = 0;
  while ((method = sel_method(opt, "Calculation of the Power", kind, method)) != 0)
  {
    switch (method)
    {
      case 1 : po_ttest();
        break;
      case 2 : po_tmcomp();
        break;
      case 3 : po_ctest(opt);
        break;
      case 4 : po_cmcomp();
        break;
      case 5 : po_life(opt);
        break;
      default : disperr(opt.pname(), "Wrong number specified !");
    }
  }
}

void designclass::caldiff(const option& opt, int kind) const // 検出差の計算
{
  int method = 0;
  while ((method = sel_method(opt, "Calculation of the Significant Difference", kind, method)) != 0)
  {
    switch (method)
    {
      case 1 : diff_ttest();
        break;
      case 2 : diff_ctest();
        break;
      case 3 : diff_life();
        break;
      default : disperr(opt.pname(), "Wrong number specified !");
    }
  }
}

void designclass::calabsp(const option& opt, int kind) const // 信頼区間の絶対精度の計算
{
  int method = 0;
  while ((method = sel_method(opt, "Calculation of the Absolute Precision of CI", kind, method)) != 0)
  {
    switch (method)
    {
      case 1 : absp_ttest();
        break;
      case 2 : absp_ctest();
        break;
      case 3 : absp_lra();
        break;
      default : disperr(opt.pname(), "Wrong number specified !");
    }
  }
}

void designclass::calaepr(const option& opt, int kind) const // 有害事象の確率計算
{
  int method = 0;
  while ((method = sel_method(opt, "Calculation of the Probability of AE", kind, method)) != 0)
  {
    switch (method)
    {
      case 1 : aepr_binom();
        break;
      case 2 : aepr_pici();
        break;
      default : disperr(opt.pname(), "Wrong number specified !");
    }
  }
}

int main(int argc, char* argv[]) // メインルーチン
{
  designclass design;           // メインクラス変数
  option opt(argv[0]);          // オプション
  try
  {
    opt.setdef();               // オプションの初期設定
    int kind = 0;
    while ((kind = design.sel(kind)) != 0)
    {
      switch (kind)
      {
      case 1 : design.caln(opt, kind);
        break;
      case 2 : design.calpower(opt, kind);
        break;
      case 3 : design.caldiff(opt, kind);
        break;
      case 4 : design.calabsp(opt, kind);
        break;
      case 5 : design.calaepr(opt, kind);
        break;
      case 9 : help(opt.pname(), USAGE);
        break;
      default : disperr(opt.pname(), "Wrong number specified !");
      }
    }
    disperr(opt.pname(), "Quit !");
  }
  catch (help_except& helpex)   // ヘルプファイル表示
  { return 0; }
  catch (option_error& opterr)  // オプション指定エラー
  { return 1; }
  catch (runtime_error& rterr)  // 実行時エラー
  {
    disperr(opt.pname(), rterr.what());
    return 2;
  }
  catch (...)                   // 予期せぬ例外
  {
    disperr(opt.pname(), "Unknown error occurred !");
    return 255;
  }
  return 0;
}
