// dmaine.C Version 7.42e  Copyleft (c) 杉本典夫
// Time-stamp: <2024-05-08 09:38:19 tac>
//
// dans共通関数を使ったdansプログラムのプロトタイプ。
//
// 使用ライブラリ:libdanse.so, libfune.so
// コンパイル法 :g++ -O -s -W dmaine dmaine.C
//              -I/usr/local/include -L/usr/local/lib -ldanse -lfune

#include <danse.H>              // dans用ヘッダファイル(mate.H含む)

using namespace dans;           // dans名前空間使用(名前空間matvec含む)

const vecstr USAGE =            // プログラム使用法
{
  "dmaine  Version 7.42e        Copyleft (Vo\\oV) 2024 Sugimoto Norio",
  "USAGE:dmaine [<OPTION> ...] [<DATA FILE> ...]",
  "  Prototype of the DANS program.",
  "<OPTION>",
  "  -c=[<START NUMBER>][-[<END NUMBER>]]:specify output case number",
  "  -dnsname[=<FILE>]:specify the file to output DANS data",
  "    default:no output  -dnsname:-dnsname=dmaine_dns.d",
  "  -f=<FILE>:reads options from the specified file",
  "  -h[=<MAN COMMAND>]:show online manual(-h:-h=man)",
  "  -ngp={data|mse}[,...]:select the data for Ngraph",
  "    data:plot of data  mse:mean and SE  default:all",
  "  -ngpname[=<FILE>]:specify the file to output Ngraph data",
  "    default:no output  -ngpname:-ngpname=dmaine_ngp.dat",
  "  -res=[-][{def|all|list|nd|stat}][,...]:select output results",
  "    -:with -, no output(-res=-:no output all)",
  "    def:same output as default  all:output all",
  "    list:data list  nd:number of data  stat:basic statistics",
  "    default:auto judgment according to data",
  "  -r<ITEM NUMBER>=[<LOWER LIMIT>][-[<UPPER LIMIT>]]:specify the lower and",
  "    upper limit for stratified items",
  "  -s<ITEM NUMBER>=<STRING>[,<STRING>,...]:specify the strings for stratify",
  "    items",
  "    output only cases that match the specified numeric range or strings",
  "    -r and -s can be mixed, and they are connected OR"
};

const string Title = "=== Prototype of the DANS Command ==="; // 表題
const string DNSNAMEdef = "dmaine_dns.d"; // 省略時DANSファイル名
const string NGPNAMEdef = "dmaine_ngp.dat"; // 省略時Ngraphファイル名

class option                    // オプションクラス
{
private:
  string vpath;                 // プログラムのパス
  string vpname;                // プログラム名
  pairint vc;                   // 対象ケース番号
  lstra vstralst;               // 層別情報リスト
  string vdnsname;              // DANSファイル名
  string vngpname;              // Ngraphファイル名
  lstr vngplst;                 // Ngraph出力リスト
  map<string, bool> vngpmap;    // Ngraph出力の有無マップ
  lstr vreslst;                 // 解析結果出力リスト
  map<string, bool> vresmap;    // 解析結果出力の有無マップ
public:
  option(const char* c_pname = nullptr); // コンストラクタ
  ~option()                              // デストラクタ
  {}
  const string& path() const    // プログラムのパスを返す
  { return vpath; }
  const string& pname() const   // プログラム名を返す
  { return vpname; }
  int cs() const                // 開始ケース番号を返す
  { return vc.first; }
  int ce() const                // 終了ケース番号を返す
  { return vc.second; }
  lstra& stralst()              // 層別情報リストを返す
  { return vstralst; }
  const string& dnsname() const // DANSデータファイル名を返す
  { return vdnsname; }
  const string& ngpname() const // Ngraphデータファイル名を返す
  { return vngpname; }
  bool ngp(const string& code); // Ngraph出力フラッグを返す
  bool res(const string& code); // 解析結果出力フラッグを返す
  void setdef()                 // 省略時オプション指定を設定する
  { getoptdef(*this); }
  void set(const string& optstr, char devc = OPTdev);
  // オプション指定によってオプションを設定する
  int check(size_t ni, size_t nc);  // オプションのチェックとセット
  int set_ngp();                // -ngpオプションのセット
  int set_res();                // -resオプションのセット
  bool smatch(datac& cdata) const // 層別のチェック
  { return stramatch(vstralst, cdata); }
};

option::option(const char* c_pname) : vpath(), vpname(), vc(1, INT_MAX), vstralst(), vdnsname(), vngpname(), vngplst(), vngpmap(), vreslst(), vresmap()
// コンストラクタ
{
  if (c_pname == nullptr) return; // 初期設定せず
  vpname = getname(c_pname);      // パス名なしのプログラム名取得
  vpath = c_pname;
  strsize_t idx = vpath.find(vpname);
  if (idx != string::npos) vpath.erase(idx); // プログラム名削除
}

bool option::ngp(const string& code) // Ngraph出力フラッグを返す
{
  map<string, bool>::iterator ngppos = vngpmap.find(code);
  if (ngppos == vngpmap.end()) return false; // code無し！
  return vngpmap[code];                     // true/falseを返す
}

bool option::res(const string& code) // 結果出力フラッグを返す
{
  map<string, bool>::iterator respos = vresmap.find(code);
  if (respos == vresmap.end()) return false; // code無し！
  return vresmap[code];                     // true/falseを返す
}

void option::set(const string& optstr, char devc)
// オプション指定によってオプションを設定する
{
  string opt[2];                // オプションのopt[0]:項目,opt[1]:内容
  if (getopt(opt, optstr, devc) < 1) return; // オプション指定なし
  int err = 0;
  if (opt[0] == "c")            // 対象ケース番号の指定
  {
    if (getrange(vc.first, vc.second, opt[1]) < 1)
    { err = badopt(vpname, optstr, USAGE, 1.0, (double)INT_MAX); }
    else if (vc.first < 1 || vc.first > vc.second)
    { err = badopt(vpname, optstr, USAGE, 1.0, (double)INT_MAX); }
  }
  else if (opt[0] == "dnsname") // DANSファイル名指定
  {
    if (opt[1].empty()) vdnsname = DNSNAMEdef;
    else vdnsname = opt[1];
  }
  else if (opt[0] == "f")       // オプション指定ファイル
  { err = fgetopt(*this, opt[1], true); }
  else if (opt[0] == "h")       // オンラインヘルプ表示
  { err = help(vpname, USAGE, opt[1]); }
  else if (opt[0] == "ngp")     // Ngraph出力項目指定
  {
    if (strsplit(vngplst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0] == "ngpname")     // Ngraphファイル名指定
  {
    if (opt[1].empty()) vngpname = NGPNAMEdef;
    else vngpname = opt[1];
  }
  else if (opt[0] == "res")     // 解析結果の出力指定
  {
    if (strsplit(vreslst, opt[1], CHRdelm) < 1) err = badopt(vpname, optstr, USAGE);
  }
  else if (opt[0][0] == 'r' || opt[0][0] == 's') // 層別指定
  {
    stra tstra;
    char* prt = nullptr;
    int ino = strtol(opt[0].substr(1).c_str(), &prt, 0);
    if (tstra.get(opt[0][0], ino, opt[1]) == datanon) err = badopt(vpname, optstr, USAGE);
    vstralst.push_back(tstra);
  }
  else                          // 間違ったオプション指定
  { err = nonopt(vpname, optstr, USAGE); }
  if (err > 0) throw help_except("Show this help message");
  if (err < 0) throw option_error(string("Option specification error->") + optstr);
}

int option::check(size_t ni, size_t nc) // オプションのチェックとセット
{
  if (vc.second == INT_MAX) vc.second = (int)nc;
  else if (vc.second > (int)nc) return disperr(vpname, "Wrong -c option !");
  if (straok(vstralst, ni) == false) return disperr(vpname, "Wrong -r or -s option !");
  if (set_ngp() < 0) return -2; // -ngpオプションのエラー
  if (set_res() < 0) return -3; // -resオプションのエラー
  return 0;                     // エラーなし
}

int option::set_ngp()           // -ngpオプションのセット
{
  vngpmap["data"] = vngpmap["mse"] = true;
  if (vngplst.size() > 0)
  {
    if (vngpname == "") return disperr(vpname, "Requires -ngpname option !");
    vngpmap["data"] = vngpmap["mse"] = false;
    for (string ngp : vngplst)
    {
      map<string, bool>::iterator ngppos = vngpmap.find(ngp);
      if (ngppos == vngpmap.end())
      { return disperr(vpname, "Wrong -ngp=" + ngp + " option !"); }
      else
      { vngpmap[ngp] = true; }
    }
  }
  return 0;                     // エラーなし
}

int option::set_res()           // -resオプションのセット
{
  vresmap["res"] = vresmap["def"] = vresmap["nd"] = true;
  vresmap["list"] = vresmap["stat"] = false;
  if (vreslst.size() > 0)
  {
    vresmap["def"] = vresmap["nd"] = false;
    for (string res : vreslst)
    {
      map<string, bool>::iterator respos = vresmap.find(res);
      if (res == "-")           // 全て出力せず
      { vresmap["res"] = false; }
      else if (res[0] == '-' && res.size() > 1) // 特定の項目を出力せず
      {
        respos = vresmap.find(res.substr(1));
        if (respos == vresmap.end())
        { return disperr(vpname, "Wrong -res=" + res + " option !"); }
        else
        { vresmap[res.substr(1)] = false; }
      }
      else if (res == "def")    // 省略時出力
      { vresmap["def"] = vresmap["nd"] = true; }
      else if (res == "all")    // 全て出力
      {
        vresmap["def"] = vresmap["nd"] = vresmap["stat"] = vresmap["list"] = true;
      }
      else if (respos == vresmap.end())
      { return disperr(vpname, "Wrong -res=" + res + " option !"); }
      else
      { vresmap[res] = true; }
    }
  }
  return 0;                     // エラーなし
}

class varclass                  // 変数クラス
{
private:
  int vnc;                       // 解析対象ケース数
  vector<statv<double>> statvec; // 統計量ベクトル
public:
  varclass(size_t ni) : vnc(0), statvec(ni + 1) // コンストラクタ
  {}
  ~varclass()                   // デストラクタ
  {}
  int cal(const option& opt, ldata& datalst);
  // データを集計して変数をセットする
  int nc() const                // 解析対象ケース数を返す
  { return vnc; }
  int n(size_t idx) const       // 例数を返す
  { return (idx < statvec.size()) ? (statvec[idx]).n() : 0; }
  double mean(size_t idx) const // 平均値を返す
  { return (idx < statvec.size()) ? (statvec[idx]).mean() : BLANKf; }
  double vr(size_t idx) const   // 不偏分散を返す
  { return (idx < statvec.size()) ? (statvec[idx]).vr() : BLANKf; }
  double se(size_t idx) const   // 標準誤差を返す
  { return (idx < statvec.size()) ? (statvec[idx]).se() : BLANKf; }
};

int varclass::cal(const option& opt, ldata& datalst)
// データを集計して変数をセットする
{
  size_t ni = statvec.size() - 1;
  vnc = 0;
  int cno = 1;
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr, ++cno)
  {
    if (opt.cs() <= cno && cno <= opt.ce() && opt.smatch(*ditr))
    {
      for (size_t j = 0; j < ni; ++j)
      {
        if (ditr->fdata(j) != BLANKf) statvec[j].sigma(ditr->fdata(j));
      }
      ++vnc;
    }
  }
  for (size_t j = 0; j < ni; ++j) statvec[ni] += statvec[j];
  return vnc;                   // 解析対象ケース数を返す
}

class dmainclass                // dansメインクラス
{
private:
  string fname;                 // ファイル名
  void put_resid(ostream& fout, idd& id) const;
  // 解析結果のタイトルとデータ識別情報を出力する
  void put_res(ostream& fout, option& opt, idd& id, ldata& datalst, const varclass& var) const;
  // 解析結果を出力する
  void put_dns(const option& opt, idd& id, ldata& datalst, const varclass& var) const;
  // DANSデータを出力する
  void plot_ngp(option& opt, idd& id, ldata& datalst, const varclass& var) const;
  // Ngraph用データを出力する
  void plot_ngpid(ofstream& fout, idd& id) const;
  // Ngraph用データのタイトルとデータ識別情報を出力する
  void plot_data(ofstream& fout, idd& id, ldata& datalst) const;
  // Ngraph用に生データを出力する
  void plot_mse(ofstream& fout, const idd& id, const varclass& var) const;
  // Ngraph用に平均値±SEを出力する
public:
  dmainclass() : fname()        // コンストラクタ
  {}
  ~dmainclass()                 // デストラクタ
  {}
  int get(const char* c_fname, option& opt, idd& id, ldata& datalst);
  // データを読み込む
  void put(option& opt, idd& id, ldata& datalst) const; // 集計結果を出力する
};

void dmainclass::put_resid(ostream& fout, idd& id) const
// 解析結果のタイトルとデータ識別情報を出力する
{
  puttitle(fout, Title);
  fout << "\n";
  id.put(fout);
}

void dmainclass::put_res(ostream& fout, option& opt, idd& id, ldata& datalst, const varclass& var) const
// 解析結果を出力する
{
  constexpr int LENline = 74; // 区切り線の長さ
  put_resid(fout, id);
  fout << "\n";
  bool putres = false;
  if (opt.res("list"))          // データリスト出力
  {
    int cno = 1;
    for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr)
    {
      if (opt.cs() <= cno && cno <= opt.ce() && opt.smatch(*ditr))
      { fout << "CASE" << setw(4) << cno << ":";  ditr->put(fout); }
      ++cno;
    }
    putres = true;
  }
  if (opt.res("nd"))            // データ数出力
  {
    if (putres) fout << "\n";
    else putres = true;
    fout << "Number of Stratifications=" << id.ns() << "  Number of Items=" << id.ni() << "  Number of Cases=" << id.nc() << "\n";
    fout << "  cs=" << opt.cs() << " ce=" << opt.ce() << "  Number of Cases to be analyzed=" << var.nc() << "\n";
  }
  if (opt.res("stat"))          // 統計量出力
  {
    if (putres) fout << "\n";
    fout << "<BASIC STATISTICS>\n";
    putline(fout, LENline);
    for (size_t j = 0; j < id.ni(); ++j)
    {
      fout << "ITEM" << setw(4) << j + 1 << ":";
      putmsd(fout, var.n(j), var.mean(j), var.vr(j));
    }
    putline(fout, LENline);
    fout << "TOTAL   :";
    putmsd(fout, var.n(id.ni()), var.mean(id.ni()), var.vr(id.ni()));
  }
}

void dmainclass::put_dns(const option& opt, idd& id, ldata& datalst, const varclass& var) const
// DANSデータを出力する
{
  ofstream fout;
  if (ofopen(fout, (opt.dnsname()).c_str()) == false) return; // ファイルオープン失敗
  id.put(fout, formtag);
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr) ditr->put(fout);
}

void dmainclass::plot_ngp(option& opt, idd& id, ldata& datalst, const varclass& var) const
// Ngraph用データ出力
{
  ofstream fout;
  if (ofopen(fout, (opt.ngpname()).c_str()) == false) return; // ファイルオープン失敗
  bool delml = false;
  if (opt.ngp("data"))          // 生データ
  {
    fout << "-s1 -zx0," << id.ni() + 1 << ",1 -dline -w40\n";
    plot_ngpid(fout, id);
    plot_data(fout, id, datalst);
    delml =true;
  }
  if (opt.ngp("mse"))           // 平均値±SE
  {
    if (delml) fout << NGPdelm << "\n";
    else fout << "-s1 -zx0," << id.ni() + 1 << ",1 -dline -w80\n";
    plot_ngpid(fout, id);
    plot_mse(fout, id, var);
  }
}

void dmainclass::plot_ngpid(ofstream& fout, idd& id) const
// Ngraph用データのタイトルとデータ識別情報を出力する
{
  fout << "%";  puttitle(fout, Title);
  fout << "%\n";
  id.put(fout, formngp);
}

void dmainclass::plot_data(ofstream& fout, idd& id, ldata& datalst) const
// Ngraph用に生データを出力する
{
  fout << "%<RAW DATA>\n";
  ldata_itr dpos = datalst.begin();
  bool putdelml = false;
  for (ldata_itr ditr = datalst.begin(); ditr != datalst.end(); ++ditr)
  {
    if (putdelml) fout << "= =\n";
    else putdelml = true;
    for (size_t j = 0; j < id.ni(); ++j)
    {
      double fdata = ditr->fdata(j);
      if (fdata != BLANKf) fout << j + 1 << " " << fdata << "\n";
    }
  }
}

void dmainclass::plot_mse(ofstream& fout, const idd& id, const varclass& var) const
// Ngraph用に平均値±SEを出力する
{
  fout << "%<MEAN AND SE>\n";
  for (size_t j = 0; j < id.ni(); ++j)
  {
    fout << j + 1 << " " << var.mean(j) << " " << var.se(j) << " " << -var.se(j) << "\n";
  }
}

int dmainclass::get(const char* c_fname, option& opt, idd& id, ldata& datalst)
// データを読み込む
{
  if (c_fname == nullptr) fname.erase();
  else fname = c_fname;
  int nc = fgetdata(fname, id, datalst);
  if (nc < 0) throw runtime_error("Run-time error !");
  if (opt.check(id.ni(), id.nc()) != 0) throw option_error("Run-time option error !");
  if (id.append_stra(opt.stralst()) < 0) throw option_error("wrong stratification !");
  return nc;                    // 読み込んだケース数を返す
}

void dmainclass::put(option& opt, idd& id, ldata& datalst) const
// 結果を出力する
{
  if (id.nc() < 1)              // データなし
  {
    if (opt.res("res"))
    {
      put_resid(cout, id);
      cout << "\n" << endl;
    }
    return;
  }
  varclass var(id.ni());        // 変数オブジェクト
  int nc = var.cal(opt, datalst); // データを集計して変数をセットする
  if (opt.res("res"))
  {
    put_res(cout, opt, id, datalst, var);
    cout << "\n" << endl;
  }
  if (opt.dnsname() != "") put_dns(opt, id, datalst, var); // DANSデータ出力
  if (opt.ngpname() != "") plot_ngp(opt, id, datalst, var); // Ngraph用データ出力
}

int main(int argc, char* argv[]) // メインルーチン
{
  dmainclass dmain;             // メインクラス変数
  option opt(argv[0]);          // オプション
  idd id;                       // データ識別情報
  ldata datalst;                // データリスト
  try                           // 例外監視
  {
    opt.setdef();               // オプションの初期設定
    bool fok = true;            // ファイル処理モードフラッグ
    for (int i = 1; i <= argc; ++i)
    {
      if (i >= argc)
      {
        if (fok) dmain.get(nullptr, opt, id, datalst); // 標準入力から読み込む
      }
      else if (*(argv[i]) == CHRopt)
      { opt.set(argv[i] + 1);  fok = true; }
      else
      { dmain.get(argv[i], opt, id, datalst);  fok = false; }
    }
    dmain.put(opt, id, datalst);
  }
  catch (help_except& helpex)   // ヘルプファイル表示
  { return 0; }
  catch (option_error& opterr)  // オプション指定エラー
  { return 1; }
  catch (runtime_error& rterr)  // 実行時エラー
  {
    disperr(opt.pname(), rterr.what());
    return 2;
  }
  catch (...)                   // 予期せぬ例外
  {
    disperr(opt.pname(), "Unknown error occurred !");
    return 255;
  }
  return 0;
}
